package com.chinacreator.datasync.handler;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.log4j.Logger;

import com.chinacreator.datasync.common.ShareVarUtil;
import com.chinacreator.datasync.entity.DataSyncJobBean;
import com.chinacreator.datasync.entity.OrderSyncKafkaBean;

import kafka.consumer.Consumer;
import kafka.consumer.ConsumerConfig;
import kafka.consumer.KafkaStream;
import kafka.javaapi.consumer.ConsumerConnector;

/**
 * 
 * kafka消费
 * 
 * @author maoya
 *
 */
public class DataSyncOrderConsumerHandler{

	private static Logger logger = Logger.getLogger("syncConsumer");

	private final BlockingQueue<OrderSyncKafkaBean> sharedQueue;
	private final DataSyncJobBean jobBean;
	private final String jobName;
	private final int threadid;
	private ConsumerConnector consumer;
	private ExecutorService executor;

	public DataSyncOrderConsumerHandler(BlockingQueue<OrderSyncKafkaBean> realTimeQueue, DataSyncJobBean jobBean,
			String jobName, int threadid) {
		this.sharedQueue = realTimeQueue;
		this.jobBean = jobBean;
		this.jobName = jobName;
		this.threadid = threadid;
	}

	public void run() {
		boolean flag = false;
		logger.info("[数据同步CONSUMER-THREAD]...初始化开始..."+ this.jobName + "_" + this.threadid + ":" +this.jobBean.getTopic());
		
		try {
			this.consumer = Consumer.createJavaConsumerConnector(createConsumerConfig(this.jobBean));
			Map<String, Integer> topicCountMap = new HashMap<String, Integer>();
			topicCountMap.put(this.jobBean.getTopic(), 1);// 一次从主题中获取一个数据 
			Map<String, List<KafkaStream<byte[], byte[]>>> consumerMap = consumer.createMessageStreams(topicCountMap);
			logger.info(this.jobName + "_" + this.threadid + ":" +this.jobBean.getTopic()+"["+this.jobBean.getGroupid()+"] , map size ==> " + consumerMap.size());
			
			List<KafkaStream<byte[], byte[]>> streams = consumerMap.get(jobBean.getTopic());
			logger.info(this.jobName + "_" + this.threadid + ":" +this.jobBean.getTopic()+" streams size>>>>>>>>>>>>>>>>>>"+streams.size());
			
			int threadNumber = 0;
			executor = Executors.newFixedThreadPool(3);
			for (final KafkaStream stream : streams) {
				threadNumber++;
				String threadName = this.jobName + "_" + this.threadid +  "[" + threadNumber + "]:" +this.jobBean.getTopic();
				executor.submit(new DataSyncOrderConsumerThread(stream, this.jobName, threadName, threadNumber, this.sharedQueue,"cdn".equals(this.jobBean.getCpid())));
				flag = true;
			}
		} catch (Exception e) {
			e.printStackTrace();
			
			logger.error("[数据同步CONSUMER-THREAD]...初始化异常..."+ this.jobName + "_" + this.threadid + ":" +this.jobBean.getTopic()+","+e.getClass().getName()+":"+e.getMessage()); 
		}
		
		if(!flag) {
			logger.info("[数据同步CONSUMER-THREAD]...初始化失败..."+ this.jobName + "_" + this.threadid + ":" +this.jobBean.getTopic());
			ShareVarUtil.setJobConsumerThreadNum(this.jobName, 0);
//			ShareVarUtil.addJobConsumerThreadNum(this.jobName,-1);
		}
	}
	
	
	private ConsumerConfig createConsumerConfig(DataSyncJobBean jobBean) {
		Properties props = new Properties();
		props.put("zookeeper.connect", jobBean.getZkServer());
		props.put("group.id", jobBean.getGroupid());
//		props.put("group.id", this.jobName);
//      props.put("consumer.id", this.jobName + "_" + this.threadid);
		props.put("auto.offset.reset", "largest");
		props.put("enable.auto.commit", "true");
		props.put("zookeeper.session.timeout.ms", "120000");
		props.put("zookeeper.sync.time.ms", "6000");
		props.put("auto.commit.interval.ms", "3000");
		props.put("rebalance.backoff.ms", "60000");
		props.put("rebalance.max.retries", "3");
		return new ConsumerConfig(props);
	}
	
}
