package com.chinacreator.datasync.handler;

import com.alibaba.fastjson.JSON;
import com.chinacreator.datasync.common.ShareVarUtil;
import com.chinacreator.datasync.entity.DataSyncJobBean;
import com.chinacreator.datasync.entity.OrderSyncKafkaBean;
import com.chinacreator.datasync.util.ParamsConfig;
import com.chinacreator.datasync.util.ThreadLimitUtil;

import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.log4j.Logger;

import java.util.*;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * kafka消费
 *
 * @author maoya
 */
public class DataSyncOrderConsumerHandlerNew {

    private static Logger logger = Logger.getLogger("syncConsumerNew");

    private final BlockingQueue<OrderSyncKafkaBean> sharedQueue;
    private final DataSyncJobBean jobBean;
    private final String jobName;
    private final int threadid;
    //private ConsumerConnector consumer;
    private KafkaConsumer<String, String> consumer;
    private ExecutorService executor;
    
    //#CDN同步内存队列长度设置
    private int queuelencdn;
    
    //#CP同步内存队列长度设置
    private int queuelencp;
    
    //kafka每次取数据的条数
    private int maxpollrecords;
    
    private String autooffsetreset;

    public DataSyncOrderConsumerHandlerNew(BlockingQueue<OrderSyncKafkaBean> realTimeQueue, DataSyncJobBean jobBean,
                                           String jobName, int threadid, int queuelencdn, int queuelencp, int maxpollrecords, String autooffsetreset) {
        this.sharedQueue = realTimeQueue;
        this.jobBean = jobBean;
        this.jobName = jobName;
        this.threadid = threadid;
        this.queuelencdn = queuelencdn;
        this.queuelencp = queuelencp;
        this.maxpollrecords = maxpollrecords;
        this.autooffsetreset = autooffsetreset;
    }

    public void run() {
        boolean flag = false;
        logger.info("[数据同步CONSUMER-THREAD]...初始化开始..." + this.jobName + "_" + this.threadid + ":" + this.jobBean.getTopic());
        try {
        	logger.info("queuelencdn=>"+queuelencdn+"，queuelencp=>"+queuelencp+"，maxpollrecords=>"+maxpollrecords+"，autooffsetreset=>"+autooffsetreset);
            //this.consumer = Consumer.createJavaConsumerConnector(createConsumerConfig(this.jobBean));
            this.consumer = init(jobBean);
            String threadkey = jobBean.getTopic();
            ThreadLimitUtil.setThread(threadkey);//设置threadkey
            ExecutorService executor = Executors.newFixedThreadPool(3);
            String threadName = this.jobName + "_" + this.threadid  +this.jobBean.getTopic();
            executor.submit(new DataSyncOrderConsumerThreadNew(consumer, this.jobBean, threadName, this.sharedQueue,"cdn".equals(this.jobBean.getCpid()), queuelencdn, queuelencp, autooffsetreset));
            flag = true;
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("[数据同步CONSUMER-THREAD]...初始化异常..." + this.jobName + "_" + this.threadid + ":" + this.jobBean.getTopic() + "," + e.getClass().getName() + ":" + e.getMessage());
        }

        if (!flag) {
            logger.info("[数据同步CONSUMER-THREAD]...初始化失败..." + this.jobName + "_" + this.threadid + ":" + this.jobBean.getTopic());
            ShareVarUtil.setJobConsumerThreadNum(this.jobName, 0);
//			ShareVarUtil.addJobConsumerThreadNum(this.jobName,-1);
        }
    }


    public KafkaConsumer<String, String> init(DataSyncJobBean conf) {
        logger.info("++++++++++++++++++++++++开始创建客户端++++++++++++");
        Properties props = new Properties();
        //在启动consumer时配置的broker地址的。不需要将cluster中所有的broker都配置上，因为启动后会自动的发现cluster所有的broker。
        //它配置的格式是：host1:port1;host2:port2…
        props.put("bootstrap.servers", conf.getKafkaServer());
        //设置GROUPID，用于表示该consumer想要加入到哪个group中。默认值是 ""
        //props.put("group.id", "test_hlx_realtime_order_topic");
        props.put("group.id", conf.getGroupid());
        //Consumer 在commit offset时有两种模式：自动提交，手动提交。手动提交在前面已经说过。自动提交：是Kafka Consumer会在后台周期性的去commit。
        //默认值是true。
        props.put("enable.auto.commit", "true");//设置自动提交
        //自动提交间隔。范围：[0,Integer.MAX]，默认值是 5000 （5 s）
        props.put("auto.commit.interval.ms", "1000");
        //Consumer session 过期时间。这个值必须设置在broker configuration中的group.min.session.timeout.ms 与 group.max.session.timeout.ms之间。
        //其默认值是：10000 （10 s）
        props.put("session.timeout.ms", "30000");
        //每次取数据条数， 一次调用poll()操作时返回的最大记录数，默认值为500
        props.put("max.poll.records", maxpollrecords); //200
        //请求发起后，并不一定会很快接收到响应信息。这个配置就是来配置请求超时时间的。默认值是：305000 （305 s）
        props.put("request.timeout.ms", "40000");

        //当consumer向一个broker发起fetch请求时，broker返回的records的大小最小值。如果broker中数据量不够的话会wait，直到数据大小满足这个条件。
        //取值范围是：[0, Integer.Max]，默认值是1。
        //默认值设置为1的目的是：使得consumer的请求能够尽快的返回。

        props.put("fetch.min.bytes", "1");
        /***
		earliest
		当各分区下有已提交的offset时，从提交的offset开始消费；无提交的offset时，从头开始消费
		latest
		当各分区下有已提交的offset时，从提交的offset开始消费；无提交的offset时，消费新产生的该分区下的数据
		none
		topic各分区都存在已提交的offset时，从offset后开始消费；只要有一个分区不存在已提交的offset，则抛出异常
         */
        // 这个配置项，是告诉Kafka Broker在发现kafka在没有初始offset，或者当前的offset是一个不存在的值（如果一个record被删除，就肯定不存在了）时，该如何处理。它有4种处理方式：
        //1） earliest：自动重置到最早的offset。
        //2） latest：看上去重置到最晚的offset。
        //3） none：如果边更早的offset也没有的话，就抛出异常给consumer，告诉consumer在整个consumer group中都没有发现有这样的offset。
        //4） 如果不是上述3种，只抛出异常给consumer。
        //默认值是latest。
        //props.put("auto.offset.reset", "earliest");//earliest,latest
        props.put("auto.offset.reset", autooffsetreset);//earliest,latest
        
        //Message record 的key, value的反序列化类。
        props.put("key.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
        props.put("value.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
        KafkaConsumer<String, String> consumer = new KafkaConsumer<String, String>(props);
        logger.info("-----------------客户端创建完成");


        //设置到内存队列
        String threadkey = conf.getTopic();
        ThreadLimitUtil.setRecordsMax(threadkey, "20"); //设置当前topic最大取数据条数
        return consumer;
    }

}
