package com.chinacreator.datasync.handler;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import org.apache.log4j.Logger;

import com.alibaba.fastjson.JSON;
import com.chinacreator.datasync.common.ShareVarUtil;
import com.chinacreator.datasync.entity.OrderSyncKafkaBean;

import kafka.consumer.ConsumerIterator;
import kafka.consumer.KafkaStream;
import kafka.message.MessageAndMetadata;

/**
 * 
 * 数据同步消费
 * 
 * @author maoya
 *
 */
public class DataSyncOrderConsumerThread  implements Runnable {
	
	private final KafkaStream stream;
    private final String jobName;
    private final String threadName;
    private final int threadNum ;
    private final boolean iscdn ;
    private final BlockingQueue<OrderSyncKafkaBean> sharedQueue;
    
    private static Logger logger = Logger.getLogger("syncConsumer");
    
    private static Logger errorlogger = Logger.getLogger("error");
    
    public DataSyncOrderConsumerThread(KafkaStream a_stream,String jobName, String threadName,int threadNum,BlockingQueue<OrderSyncKafkaBean> realTimeQueue,boolean iscdn) {
    	this.stream = a_stream;
    	this.jobName = jobName;
    	this.threadName = threadName;
    	this.threadNum = threadNum;
    	this.iscdn = iscdn;
        this.sharedQueue = realTimeQueue;
    }
    
	@Override
	public void run() {
    	logger.info("[数据同步CONSUMER-THREAD]...Starting up..."+ threadName);
	    try {
	    	ConsumerIterator<byte[], byte[]> it = stream.iterator();
	    	
	    	while (it.hasNext()){
	        	MessageAndMetadata<byte[], byte[]> element = it.next();
	        	OrderSyncKafkaBean orderInfo = (OrderSyncKafkaBean)JSON.parseObject(new String(element.message()), OrderSyncKafkaBean.class);
	        	long offset = element.offset();
				if (orderInfo.getValid() == 0) {
					orderInfo.setValid(offset);
				}
				
				logger.info("DATA "+this.threadName+" " + JSON.toJSONString(orderInfo));
				
				// 加入队列
				sharedQueue.add(orderInfo);
//				if(sharedQueue.size() >= (this.iscdn?1000:50)){
//					try {
//						logger.info(threadName + " Queue size >= "+(this.iscdn?1000:50)+",sleeping 50ms .... .... ");
//						Thread.currentThread().sleep(50);
//					} catch (Exception e) {}
//				}	
				
				//队列是不是数据不是太多
				while(sharedQueue.size() >= (this.iscdn?100:50)){
					try {
						logger.info(threadName + " Queue size("+sharedQueue.size()+") >= "+(this.iscdn?100:50)+",sleeping 50ms .... .... ");
						Thread.currentThread().sleep(50);
					} catch (Exception e) {}
				}	        	
				
	        }
		} catch (Exception e) {
			errorlogger.error("[数据同步CONSUMER-THREAD]...异常结束..."+ threadName+","+e.getClass().getName()+":"+e.getMessage()); 
			logger.error("[数据同步CONSUMER-THREAD]...异常结束..."+ threadName+","+e.getClass().getName()+":"+e.getMessage()); 
		}
	    
	    logger.info("[数据同步CONSUMER-THREAD]...End thread..."+ threadName);
	    ShareVarUtil.setJobConsumerThreadNum(this.jobName, 0);
//	    ShareVarUtil.addJobConsumerThreadNum(this.jobName,-1);
	}
	
}
