package com.chinacreator.datasync.handler;

import com.alibaba.fastjson.JSON;
import com.chinacreator.datasync.common.ShareVarUtil;
import com.chinacreator.datasync.dao.QueryDbDao;
import com.chinacreator.datasync.entity.DataSyncJobBean;
import com.chinacreator.datasync.entity.OrderSyncKafkaBean;
import com.chinacreator.datasync.util.SpringContextUtils;
import com.chinacreator.datasync.util.ThreadLimitUtil;

import kafka.consumer.ConsumerIterator;
import kafka.consumer.KafkaStream;
import kafka.message.MessageAndMetadata;

import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.log4j.Logger;

import java.util.Arrays;
import java.util.concurrent.BlockingQueue;

/**
 * 数据同步消费
 *
 * @author maoya
 */
public class DataSyncOrderConsumerThreadNew implements Runnable {

	private final KafkaConsumer<String, String> consumer;
	private final DataSyncJobBean jobBean;
	private final String threadName;
	private final boolean iscdn;
	private final BlockingQueue<OrderSyncKafkaBean> sharedQueue;
	long reTime = 1 * 30; //刷新问隔, 单位是秒
	
    //#CDN同步内存队列长度设置
    private int queuelencdn;
    
    //#CP同步内存队列长度设置
    private int queuelencp;

    private String autooffsetreset;
    
	private static Logger logger = Logger.getLogger("syncConsumerNew");

	private static Logger errorlogger = Logger.getLogger("error");

	public DataSyncOrderConsumerThreadNew(KafkaConsumer<String, String> consumer, DataSyncJobBean jobBean, String threadName, BlockingQueue<OrderSyncKafkaBean> realTimeQueue, 
			boolean iscdn, int queuelencdn, int queuelencp, String autooffsetreset) {
		this.consumer = consumer;
		this.jobBean = jobBean;
		this.threadName = threadName;
		this.iscdn = iscdn;
		this.sharedQueue = realTimeQueue;
        this.queuelencdn = queuelencdn;
        this.queuelencp = queuelencp;
        this.autooffsetreset = autooffsetreset;
	}

	@Override
	public void run() {
		String topic = jobBean.getTopic();
		logger.info("topic=>"+topic+"[数据同步CONSUMER-THREAD]...Starting up..." + threadName);

		try {
			consumer.subscribe(Arrays.asList(jobBean.getTopic()));
			long exeTime = System.currentTimeMillis()/1000; //当前刷新时间
			while(true) {
				ConsumerRecords<String, String> records = consumer.poll(100);
				//logger.info(topic+"获取数据条数"+(records == null ? "0" : records.count()));
				exeTime = beforeProc(exeTime, jobBean, records);
				if (records.count() > 0) {
					for (ConsumerRecord<String, String> record : records) {
						long offset = record.offset();
						String dataJson = record.value();
						int partition = record.partition();
						OrderSyncKafkaBean orderInfo = (OrderSyncKafkaBean) JSON.parseObject(dataJson, OrderSyncKafkaBean.class);
						if (orderInfo.getValid() == 0) {
							orderInfo.setValid(offset);
						}
						logger.info("DATA topic=>" + topic + "，partition=>"+ partition + " " + JSON.toJSONString(orderInfo));
						// 加入队列
						sharedQueue.add(orderInfo);
						//队列是不是数据不是太多
						while (sharedQueue.size() >= (this.iscdn ? queuelencdn : queuelencp)) {
							try {
								logger.info(threadName + " 内存队列积压，Queue size(" + sharedQueue.size() + ") >= " + (this.iscdn ? queuelencdn : queuelencp) + ",sleeping 50ms .... .... ");
								Thread.currentThread().sleep(50);
							} catch (Exception e) {
							}
						}
						ThreadLimitUtil.sharedQueueMap.put(jobBean.getTopic()+"_queue",false);
						//logger.info("------存放数据");
						logger.info(ThreadLimitUtil.sharedQueueMap.get(jobBean.getTopic()+"_queue"));
					}
				} else {
					//logger.info(threadName+"----------------无数据");
				}
				afterProc(jobBean);
			}
		} catch (Exception e) {
			errorlogger.error("[数据同步CONSUMER-THREAD]...异常结束..." + threadName + "," + e.getClass().getName() + ":" + e.getMessage());
			logger.error("[数据同步CONSUMER-THREAD]...异常结束..." + threadName + "," + e.getClass().getName() + ":" + e.getMessage());
		}

		logger.info("[数据同步CONSUMER-THREAD]...End thread..." + threadName);
		ShareVarUtil.setJobConsumerThreadNum(this.jobBean.getName(), 0);
//	    ShareVarUtil.addJobConsumerThreadNum(this.jobName,-1);
	}


	/**
	 * 执行之前
	 * @param exeTime 当前执行时间
	 * @param reTime  刷新间隔
	 * @param logger  日志
	 * @param topic
	 * @param records
	 */
	public long beforeProc(long exeTime, DataSyncJobBean jobBean, ConsumerRecords<String, String> records){
		try{
			long currTime = System.currentTimeMillis()/1000;
			//logger.info("exeTime=>"+exeTime +" , currTime=>"+ currTime+", reTime=>"+ reTime);
			//控制写日志频率
			if((currTime - exeTime) > reTime){
				logger.info("topicname="+jobBean.getTopic()+"，recordscount======="+records.count());
				exeTime = currTime;
			}
		}catch(Exception e){

		}
		return exeTime;
	}

	/**
	 * 处理数据之后要执行流程
	 * @return
	 */
	public boolean afterProc(DataSyncJobBean jobBean){
		boolean res = false;
		String topic = jobBean.getTopic();
		String groupid = jobBean.getGroupid();
		String threadkey = topic;
		ThreadLimitUtil.setConsLive(threadkey);
		//如果当前线程数为0，或者大于1，则退出当前线程
		if(ThreadLimitUtil.getThread(threadkey) == 0 || ThreadLimitUtil.getThread(threadkey) > 3){
			ThreadLimitUtil.removeKey(threadkey);
			res = true;
		}
		return res;
	}

}

