package com.chinacreator.datasync.handler;

import java.net.URLEncoder;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.BlockingQueue;

import oracle.net.aso.f;
import oracle.net.aso.s;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.aop.ThrowsAdvice;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.datasync.common.CommonUtil;
import com.chinacreator.datasync.common.ConfigUtil;
import com.chinacreator.datasync.common.ShareVarUtil;
import com.chinacreator.datasync.dao.QueryDbDao;
import com.chinacreator.datasync.entity.DataSyncJobBean;
import com.chinacreator.datasync.entity.OrderCdnSyncBean;
import com.chinacreator.datasync.entity.OrderCpSyncBean;
import com.chinacreator.datasync.entity.OrderCpSyncBeanYoutu;
import com.chinacreator.datasync.entity.OrderSyncKafkaBean;
import com.chinacreator.datasync.entity.OrderSyncKafkaBeanResult;
import com.chinacreator.datasync.entity.SyncOrderBean;
import com.chinacreator.datasync.kafka.KafkaProductor;
import com.chinacreator.datasync.util.DesUtil;
import com.chinacreator.datasync.util.HttpInvoke;
import com.chinacreator.datasync.util.MD5;
import com.chinacreator.datasync.util.ParamsConfig;
import com.chinacreator.datasync.util.SpringContextUtils;
import com.chinacreator.datasync.util.URLUtil;

/**
 *
 *
 * 数据同步
 *
 * @author maoya
 *
 */
public class DataSyncOrderSendHandler implements IDataSyncOrderJobHandler {

	public Logger errorlogger = LoggerFactory.getLogger("error");

	public Logger logger = LoggerFactory.getLogger("syncSend");

	public Logger cdnLogger = LoggerFactory.getLogger("syncCdn");

	//cdn同步参数
	public static final String url = "http://api.vsdn.wo186.tv:8088/api/orders/orders.php";
	public static final String key = "dk1ej2de3ks5c!cm%sw*order";
	public static final int timeout = 10000;


	private QueryDbDao dao;

	private final BlockingQueue<OrderSyncKafkaBean> sharedQueue;
	private final DataSyncJobBean jobBean;
	private final String jobName;
	private final int threadid;

	public DataSyncOrderSendHandler(BlockingQueue<OrderSyncKafkaBean> realTimeQueue, DataSyncJobBean jobBean,
			String jobName, int threadid, QueryDbDao dao) {

		this.sharedQueue = realTimeQueue;
		this.jobBean = jobBean;
		this.jobName = jobName;
		this.threadid = threadid;
		this.dao = dao;
	}

	/**
	 * List排序
	 * @param list
	 * @return
	 */
	private List<SyncOrderBean> listSort(List<SyncOrderBean> list) {
		logger.info("list 排序前=>"+list);
	    Collections.sort(list, new Comparator<SyncOrderBean>() {
	        @Override
	        public int compare(SyncOrderBean o1, SyncOrderBean o2) {
	            //降序，o1,o2反过来则升序
	            return o2.getCompEndtime().compareTo(o1.getCompEndtime());
	        }
	    });
	    logger.info("list 排序后=>"+list);
	    return list;
	}

	/**
	 * 获取指定时间的下个月最后一天
	 * 退订时间或结束时间或订购时间为空，不处理
	 * 订购时间小于2021年7月25日，不处理
	 * 订购日号小于25号，不处理
	 * @param endtime 结束时间
	 * @param canceltime 退订时间
	 * @param ordertime 订购时间
	 * @return
	 */
	private String getNextEndDay(String endtime, String canceltime, String ordertime){
		//退订时间为空，或者退订时间不是14位
		if(canceltime == null || canceltime.length() != 14
				|| endtime == null || endtime.length() != 14
				|| ordertime == null || ordertime.length() != 14){
			return endtime;
		}
		//活动开始时间
		String startdate = "20210725000000";
		//订购日期在活动开始日期之前，不处理
		if(Long.parseLong(startdate) > Long.parseLong(ordertime)){
			return endtime;
		}
		//设置为当前的结束时间
		String res = endtime;
		SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");
		try {
			//判断是否25号以后订购的
			Date orderDate = df.parse(ordertime);
			Calendar orderCal = Calendar.getInstance();
			orderCal.setTime(orderDate);
			//获取日号
			int days = orderCal.get(Calendar.DATE);
			if(days < 25){
				return endtime;
			}

			//ENDTIME加一个月
			df = new SimpleDateFormat("yyyyMMddHHmmss");
			Date endDate = df.parse(endtime);
			Calendar calendar = Calendar.getInstance();
	        calendar.setTime(endDate);
	        calendar.add(Calendar.MONTH, 2);
	        calendar.set(Calendar.DATE, 0);

	        res = df.format(calendar.getTime());
		} catch (Exception e) {
			res = endtime;
			e.printStackTrace();
			logger.error("获取指定时间的下个月最后一天出现异常，endtime=>"+endtime+" , canceltime=>"+canceltime+", ordertime=>"+ordertime,e);
		}

		return res;
	}

	/**
	 * 获取ENDTIME最大的订购关系，如果有赠送流量的业务退订，则会把对应的订购关系的ENDTIME增加一个月
	 * @param list
	 * @param noambArray
	 * @return
	 */
	private SyncOrderBean getEndMax(List<SyncOrderBean> list, List<String> noambArray){
		if(noambArray == null){
			noambArray = new ArrayList<String>();
		}
		//默认选一条数据
		SyncOrderBean bo = list.get(0);

		//第一条数据的结束时间 为空，不处理结果信息，不为空，则处理结果信息
		if(bo.getEndtime() != null && !"".equals(bo.getEndtime()) && bo.getEndtime().length() == 14){
			//取到数据后要把数据重新解析再排序
			//1. 查询是否有退订时传了多一个月ENDTIME的业务
			//2. 根据ENDTIME排序，ENDTIME为空的排在前面
			for(SyncOrderBean sobBean : list){
				//订购关系是要赠送的业务且当前是退订
				if(sobBean.getEndtime() != null
						&& !"".equals(sobBean.getEndtime()) //结束时间不为空
						&& ConfigUtil.isSendflowproduct(sobBean.getSpid(),dao) //是否是赠送一个月流量的产品
						&& sobBean.getCanceltime() != null
						&& !"".equals(sobBean.getCanceltime()) //退订时间不为空
						){//此业务是要赠送一个月流量的
					//ENDTIME加一个月
					sobBean.setEndtime(getNextEndDay(sobBean.getEndtime(), sobBean.getCanceltime(), sobBean.getOrdertime()));
				}
			}
			//按endtime排序，倒序，ENDTIME最大的排在前面
			List<SyncOrderBean> sortlist = listSort(list);
			//最第一个订购关系（ENDTIME最大的）
			bo = sortlist.get(0);
		}
		return bo;
	}

	/**
	 * 更新TD_DATA_SYNC_WAIT表同步结果
	 * @param syncOrderInfo
	 * @param syncstatus 同步状态
	 * @param syncinfo 同步结果
	 * @param hascdn 是否CDN同步，true是，false不是
	 */
	private void updateSyncLog(OrderSyncKafkaBean syncOrderInfo, String syncstatus, String syncinfo, boolean hascdn){
		try {
			//还原CPID，用于更新数据和数据重试
			//restoreCpid(syncOrderInfo);
			//还原同步前的参数，否则更新不了表数据状态
			restoreSyncParams(syncOrderInfo);
			//更新TD_DATA_SYNC_WAIT表为同步成功
			dao.updateSyncLog(syncOrderInfo.getSyncid(), syncOrderInfo.getUserid(), syncOrderInfo.getSynctype(), syncOrderInfo.getCpid(),syncstatus,syncinfo);
		} catch (Exception e) {
			e.printStackTrace();
			CommonUtil.sleep(100L,logger);
			try {
				//更新TD_DATA_SYNC_WAIT表为同步成功
				dao.updateSyncLog(syncOrderInfo.getSyncid(), syncOrderInfo.getUserid(), syncOrderInfo.getSynctype(), syncOrderInfo.getCpid(),syncstatus,syncinfo);
			} catch (Exception e2) {
				errorlogger.error("UPDATE TD_DATA_SYNC_WAIT 失败",e);
				if(hascdn){
					cdnLogger.info("UPDATE TD_DATA_SYNC_WAIT 失败 ,"+e.getClass().getName()+":"+e.getMessage());
				}else{
					logger.info("UPDATE TD_DATA_SYNC_WAIT 失败 ,"+e.getClass().getName()+":"+e.getMessage());
				}
			}
		}
	}

	/**
	 * 更新TD_FAKEIDS_SYNC表同步结果
	 * @param syncOrderInfo
	 * @param syncstatus 同步状态
	 * @param syncinfo 同步结果
	 * @param hascdn 是否CDN同步，true是，false不是
	 */
	private void updateFakeidsSyncLog(OrderSyncKafkaBean syncOrderInfo, String syncstatus, String syncinfo, boolean hascdn){
		try {
			//还原CPID，用于更新数据和数据重试
			//restoreCpid(syncOrderInfo);
			//还原同步前的参数，否则更新不了表数据状态
			restoreSyncParams(syncOrderInfo);
			//更新TD_FAKEIDS_SYNC表为同步成功
			dao.updateFakeidsSync(syncOrderInfo.getSyncid(), syncOrderInfo.getUserid(), syncOrderInfo.getSynctype(), syncOrderInfo.getCpid(),syncstatus,syncinfo);
		} catch (Exception e) {
			e.printStackTrace();
			CommonUtil.sleep(100L,logger);
			try {
				//更新TD_FAKEIDS_SYNC表为失败
				dao.updateFakeidsSync(syncOrderInfo.getSyncid(), syncOrderInfo.getUserid(), syncOrderInfo.getSynctype(), syncOrderInfo.getCpid(),syncstatus,syncinfo);
			} catch (Exception e2) {
				errorlogger.error("UPDATE TD_FAKEIDS_SYNC 失败",e);
					logger.info("UPDATE TD_FAKEIDS_SYNC 失败 ,"+e.getClass().getName()+":"+e.getMessage());
			}
		}
	}

	/***
	 * 设置同步数据的ENDTIME
	 * 同步业务为要赠送一个月的流量且是退订操作，订购日号在25号以后，在结束时间上加一个月
	 * @param syncOrderInfo
	 * @param noambArray
	 */
	private void setSyncOrderInfoEndtime(OrderSyncKafkaBean syncOrderInfo, List<String> noambArray){
		if(noambArray == null){
			noambArray = new ArrayList<String>();
		}
		//判断CP、CDN同步是否退订
		//CP同步：type	 操作类型 0 订购，1 退订
		//CDN同步：ordertype 操作类型 0 订购，1 退订, jobBean.getCpid()除了cdn就是其他cpid
		if((!"cdn".equals(jobBean.getCpid()) && "1".equals(syncOrderInfo.getType()))
				|| ("cdn".equals(jobBean.getCpid()) && "1".equals(syncOrderInfo.getOrdertype()))) {//是退订
			String userid = syncOrderInfo.getUserid();
			String cpid = syncOrderInfo.getCpid();
			String spid = syncOrderInfo.getSpid();
			String fromspid = syncOrderInfo.getFromSpid();
			//判断同步业务是否赠送一个月流量
			if(ConfigUtil.hasDirecBusi(fromspid,dao)
					&& noambArray != null && !noambArray.contains(fromspid)){
				String endtime = syncOrderInfo.getEndtime();
				String canceltime = syncOrderInfo.getCanceltime();
				String ordertime = syncOrderInfo.getOrdertime();
				if(endtime != null && endtime.length() == 14){//有效的结束日志，加一个月
					//获取指定日期的下月最后一天
					String res = getNextEndDay(endtime, canceltime, ordertime);
					syncOrderInfo.setEndtime(res);
					if(!endtime.equals(res)){//如果ENDTIME有更新，则保存旧的ENDTIME
						syncOrderInfo.setOldEndtime(endtime);
					}
					logger.info(userid+"，"+fromspid+"=>退订要赠送一个月流量，更新ENDTIME增加一个月。新ENDTIME=>"+res);
				}
			}
		}
	}

	/**
	 * 替换CPID，某些产品需要同步另外一个CPID
	 * @param syncOrderInfo
	 * @param replacecpidbusi
	 */
	private void replaceCpid(OrderSyncKafkaBean syncBean, String replacecpidbusi){

		try {
			if(!StringUtils.isEmpty(replacecpidbusi)){
				String spid = syncBean.getSpid();
				String cpid = syncBean.getCpid();
				//多组数据用#分隔
				String [] busiArray = replacecpidbusi.split("#");
				for(String tmp : busiArray){
					if(!StringUtils.isEmpty(tmp)){
						//匹配数据用|分隔，左边是SPID，右边是CPID
						if(spid.equals(tmp.split("\\|")[0])){//SPID相同，取CPID替换
							syncBean.setCpid(tmp.split("\\|")[1]);
							syncBean.setOldCpid(cpid); //保存原CPID，用于还原
							logger.info(syncBean.getUserid()+"，"+syncBean.getSpid()+"，"+syncBean.getCpid()+"，替换CPID成功");
							break;
						}
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error(syncBean.getUserid()+"，"+syncBean.getSpid()+", 替换CPID，某些产品需要同步另外一个CPID，出现异常，"+e.getMessage());
		}
	}

	/**
	 * 还原CPID
	 * @param syncBean
	 */
	/***
	private void restoreCpid(OrderSyncKafkaBean syncBean){
		//还原CPID，用于更新数据和数据重试
		if(!StringUtils.isEmpty(syncBean.getOldCpid()) && !"null".equals(syncBean.getOldCpid())){
			String oldcpid = syncBean.getOldCpid();
			syncBean.setCpid(oldcpid);
			syncBean.setOldCpid(null);
			logger.info(syncBean.getUserid()+"，"+syncBean.getSpid()+"，"+syncBean.getCpid()+"，还原CPID成功");
		}
	}
	***/

	/**
	 * 还原之前变更的参数，用于同步失败后数据重试
	 * @param syncBean
	 */
	private void restoreSyncParams(OrderSyncKafkaBean syncBean){
		boolean hasupdsync = false;
		//logger.info("还原参数前=>"+JSON.toJSONString(syncBean));
		//还原CPID，用于更新数据和数据重试
		if(!StringUtils.isEmpty(syncBean.getOldCpid()) && !"null".equals(syncBean.getOldCpid())){
			String oldcpid = syncBean.getOldCpid();
			syncBean.setCpid(oldcpid);
			syncBean.setOldCpid(null);
			hasupdsync = true;
		}

		//还原SPID，用于更新数据和数据重试
		if(!StringUtils.isEmpty(syncBean.getOldSpid()) && !"null".equals(syncBean.getOldSpid())){
			String oldspid = syncBean.getOldSpid();
			syncBean.setSpid(oldspid);
			syncBean.setOldSpid(null);
			hasupdsync = true;
		}

		//还原ENDTIME，用于更新数据和数据重试
		if(!StringUtils.isEmpty(syncBean.getOldEndtime()) && !"null".equals(syncBean.getOldEndtime())){
			String oldendtime = syncBean.getOldEndtime();
			syncBean.setEndtime(oldendtime);
			syncBean.setOldEndtime(null);
			hasupdsync = true;
		}

		if(hasupdsync){
			logger.info("还原参数后=>"+JSON.toJSONString(syncBean));
		}
	}

	@Override
	public void run() {
		logger.info("[数据同步SEND-THREAD]...Starting up..."+ this.jobName + "_" + this.threadid + ":" +this.jobBean.getTopic()+"（"+this.threadid+"）");

		//退订不赠送流量的SPID
		String notaddmonthbusi = ParamsConfig.notaddmonthbusi;
		//要替换CPID的产品，SPID和CPID用|分隔，多组用#分隔
		String replacecpidbusi = ParamsConfig.replacecpidbusi;
		//退订赠送一个月流量开关，0使用，其他值为禁用
		String setSendflowswitch = ParamsConfig.sendflowswitch;
		//退订不赠送流量的SPID转为LIST
		List<String> noambArray = null;
		if(notaddmonthbusi != null && !"".equals(notaddmonthbusi)){
			noambArray = Arrays.asList(notaddmonthbusi.split(","));
		}

		try {
			OrderSyncKafkaBean syncOrderInfo = null;
			boolean hassync = false; //是否已同步
			while (true) {
				hassync = false; //标识初始化
				//开始时间
				long startDealTime = System.currentTimeMillis();

				// 读出队列
				syncOrderInfo = sharedQueue.poll();
				if (syncOrderInfo != null) {
					//替换CPID
					replaceCpid(syncOrderInfo, replacecpidbusi);
					String userid = syncOrderInfo.getUserid();
					String cpid = syncOrderInfo.getCpid();
					String spid = syncOrderInfo.getSpid();

					//CP同步， jobBean.getCpid()只有cdn或其他cpid两种值
					if(!"cdn".equals(jobBean.getCpid())) {
						//判断当前要同步的CPID和SPID是否有关联业务配置
						if(CommonUtil.have(ConfigUtil.queryFhHxCpid(dao), cpid) && CommonUtil.have(ConfigUtil.queryFhHxSpid(cpid,dao), spid)) {
							//查询关联业务的订购数据
							List<SyncOrderBean> list =dao.getSyncOrderBeanList(userid, cpid, spid);
							//有关联业务的订购数据
							if(list != null && list.size()>0){
								hassync = true; //设置已有同步标识
								SyncOrderBean bo = list.get(0);

								//第一条数据的结束时间 为空，不处理结果信息，不为空，则处理结果信息
								//订购的可能是后向产品，要取到最大的结束日期
								if(bo.getEndtime() != null && !"".equals(bo.getEndtime()) && bo.getEndtime().length() == 14 && "0".equals(setSendflowswitch)){
									bo = getEndMax(list, noambArray);
								}

								//如果SPID变更了，设置保存旧SPID
								if(!spid.equals(bo.getTospid())){
									syncOrderInfo.setOldSpid(spid);
								}

								//订购
								if("0".equals(syncOrderInfo.getType())){
									syncOrderInfo.setSpid(bo.getTospid());
									syncOrderInfo.setEndtime(bo.getEndtime()==null?"":bo.getEndtime());

									//优酷后向同步两条记录，sptype:0包月（前向），1后向/按次
									//同步一条订购，再同步一条退订
									if("1".equals(bo.getSptype()) && "youtu".equals(bo.getTocpid()) && bo.getEndtime()!=null && !"".equals(bo.getEndtime())) {
										//同步订购
										syncOrderInfo.setEndtime("");
										//同步一条订购
										cpDoProcess(syncOrderInfo, this.jobBean, false);

										CommonUtil.sleep(10L, logger);

										//同步退订
										OrderSyncKafkaBean cancel = new OrderSyncKafkaBean();
										cancel.setSyncid(syncOrderInfo.getSyncid());
										cancel.setSynctype(syncOrderInfo.getSynctype());
										cancel.setArea(syncOrderInfo.getArea());
										cancel.setChannelcode(syncOrderInfo.getChannelcode());
										cancel.setCostTime(syncOrderInfo.getCostTime());
										cancel.setCpid(syncOrderInfo.getCpid());
										cancel.setDatatype(syncOrderInfo.getDatatype());
										cancel.setFakeid(syncOrderInfo.getFakeid());
										cancel.setFromCpid(syncOrderInfo.getFromCpid());
										cancel.setFromSpid(syncOrderInfo.getFromSpid());
										cancel.setId(syncOrderInfo.getId());
										cancel.setOrdertime(syncOrderInfo.getOrdertime());
										cancel.setOrdertype(syncOrderInfo.getOrdertype());
										cancel.setProvince(syncOrderInfo.getProvince());
										cancel.setSpid(bo.getTospid());
										cancel.setSynctimes(syncOrderInfo.getSynctimes());
										cancel.setSyncTimestamp(syncOrderInfo.getSyncTimestamp());
										cancel.setUserid(syncOrderInfo.getUserid());
										cancel.setUsermob(syncOrderInfo.getUsermob());
										cancel.setValid(syncOrderInfo.getValid());
										cancel.setVideoid(syncOrderInfo.getVideoid());
										cancel.setCanceltime(new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()));
										cancel.setEndtime(bo.getEndtime());
										cancel.setType("1");
										//同步一条退订
										cpDoProcess(cancel, this.jobBean, true);
									} else{//不是优酷后向产品 且不是伪码同步的topic
										cpDoProcess(syncOrderInfo, this.jobBean,true);
									}
								}else{//退订
									//当前订购包月状态，不同步退订
									if(bo.getEndtime() == null || "".equals(bo.getEndtime())){
										if(jobBean.getTopic().contains("fakeids")){
											updateFakeidsSyncLog(syncOrderInfo, "3", "合并产品无需同步", false);
										}else {
											updateSyncLog(syncOrderInfo, "0", "合并产品无需同步", false);
										}
										/***
										try {
											//更新TD_DATA_SYNC_WAIT表为同步成功
											dao.updateSyncLog(syncOrderInfo.getSyncid(), syncOrderInfo.getUserid(), syncOrderInfo.getSynctype(), syncOrderInfo.getCpid(),"0","合并产品无需同步");
										} catch (Exception e) {
											CommonUtil.sleep(100L,logger);
											try {
												//更新TD_DATA_SYNC_WAIT表为同步成功
												dao.updateSyncLog(syncOrderInfo.getSyncid(), syncOrderInfo.getUserid(), syncOrderInfo.getSynctype(), syncOrderInfo.getCpid(),"0","合并产品无需同步");
											} catch (Exception e2) {
												errorlogger.error("UPDATE TD_DATA_SYNC_WAIT 失败",e);
												logger.info("UPDATE TD_DATA_SYNC_WAIT 失败 ,"+e.getClass().getName()+":"+e.getMessage());
											}
										}
										*****/
									}else{//第一条订购关系的ENDTIME不为空
										syncOrderInfo.setSpid(bo.getTospid());
										syncOrderInfo.setEndtime(bo.getEndtime()==null?"":bo.getEndtime());
										cpDoProcess(syncOrderInfo, this.jobBean,true);
									}
								}
							}/***else{//无关联业务的订购数据
								//设置ENDTIME
								setSyncOrderInfoEndtime(syncOrderInfo,noambArray);
								cpDoProcess(syncOrderInfo, this.jobBean,true);
							}****/
						}/****else{//TB_SYNC_SP_CONFIG表没有对应记录，直接按CPID和SPID同步
							//设置ENDTIME
							setSyncOrderInfoEndtime(syncOrderInfo,noambArray);
							cpDoProcess(syncOrderInfo, this.jobBean,true);
						}****/

						//TB_SYNC_SP_CONFIG表没有对应记录，或者无关联业务的订购数据，直接按CPID和SPID同步
						if(!hassync){
							//设置ENDTIME
							setSyncOrderInfoEndtime(syncOrderInfo,noambArray);
							cpDoProcess(syncOrderInfo, this.jobBean,true);
						}

						logger.info("[send-data-cp] " +this.jobName + "_" + this.threadid + ":" +this.jobBean.getTopic()+" "+ JSON.toJSONString(syncOrderInfo)+" ,"+(System.currentTimeMillis()-startDealTime)+"ms");
					}else {//CDN同步
						//判断当前要同步的CPID和SPID是否有关联业务配置
						if(CommonUtil.have(ConfigUtil.queryFhHxCpid(dao), cpid) && CommonUtil.have(ConfigUtil.queryFhHxSpid(cpid,dao), spid)) {
							//查询关联业务的订购数据
							List<SyncOrderBean> list =dao.getSyncOrderBeanList(userid, cpid, spid);
							//有关联业务的订购数据
							if(list != null && list.size()>0){
								hassync = true; //设置已有同步标识
								SyncOrderBean bo = list.get(0);

								//第一条数据的结束时间 为空，不处理结果信息，不为空，则处理结果信息
								//订购的可能是后向产品，要取到最大的结束日期
								if(bo.getEndtime() != null && !"".equals(bo.getEndtime()) && bo.getEndtime().length() == 14 && "0".equals(setSendflowswitch)){
									bo = getEndMax(list, noambArray);
								}

								//如果SPID变更了，设置保存旧SPID
								if(!spid.equals(bo.getTospid())){
									syncOrderInfo.setOldSpid(spid);
								}

								//订购
								if("0".equals(syncOrderInfo.getOrdertype())){
									syncOrderInfo.setSpid(bo.getTospid());
									syncOrderInfo.setEndtime(bo.getEndtime()==null?"":bo.getEndtime());
									this.cdnDoProcess(syncOrderInfo, this.jobBean);
								}else{//退订
									//当前订购包月状态，不同步退订
									if(bo.getEndtime() == null || "".equals(bo.getEndtime())){
										updateSyncLog(syncOrderInfo, "0", "合并产品无需同步", true);
										/****
										try {
											dao.updateSyncLog(syncOrderInfo.getSyncid(), syncOrderInfo.getUserid(), syncOrderInfo.getSynctype(), syncOrderInfo.getCpid(),"0","合并产品无需同步");
										} catch (Exception e) {
											CommonUtil.sleep(100L,logger);
											try {
												dao.updateSyncLog(syncOrderInfo.getSyncid(), syncOrderInfo.getUserid(), syncOrderInfo.getSynctype(), syncOrderInfo.getCpid(),"0","合并产品无需同步");
											} catch (Exception e2) {
												errorlogger.error("UPDATE TD_DATA_SYNC_WAIT 失败",e);
												cdnLogger.info("UPDATE TD_DATA_SYNC_WAIT 失败 ,"+e.getClass().getName()+":"+e.getMessage());
											}
										}
										****/
									}else{//第一条订购关系的ENDTIME不为空
										syncOrderInfo.setSpid(bo.getTospid());
										syncOrderInfo.setEndtime(bo.getEndtime()==null?"":bo.getEndtime());
										this.cdnDoProcess(syncOrderInfo, this.jobBean);
									}
								}
							}/***else{//无关联业务的订购数据
								this.cdnDoProcess(syncOrderInfo, this.jobBean);
							}****/
						}/****else{//当前要同步的CPID和SPID无关联业务
							this.cdnDoProcess(syncOrderInfo, this.jobBean);
						}****/

						//TB_SYNC_SP_CONFIG表没有对应记录，或者无关联业务的订购数据，直接按CPID和SPID同步
						if(!hassync){
							//设置ENDTIME
							setSyncOrderInfoEndtime(syncOrderInfo,noambArray);
							this.cdnDoProcess(syncOrderInfo, this.jobBean);
						}

						cdnLogger.info("[send-data-cdn] " +this.jobName + "_" + this.threadid + ":" +this.jobBean.getTopic()+" "+ JSON.toJSONString(syncOrderInfo)+" ,"+(System.currentTimeMillis()-startDealTime)+"ms");
					}

					CommonUtil.sleep(1L,logger);
				}else {
					CommonUtil.sleep(500L,logger);
				}


				// 判断线程是否需要关闭
				try {
					if (this.threadid > this.jobBean.getSendThreadNum()) {
						errorlogger.error("[数据同步SEND-THREAD]...线程数异常..."+ this.jobName + "_" + this.threadid + ":" +this.jobBean.getTopic()+" 运行线程数（"+this.threadid+"） > 配置线程数（"+this.jobBean.getSendThreadNum()+"）");
						break;
					}
				} catch (Exception e) {
					CommonUtil.sleep(5000L,logger);
					continue;
				}

			}

		} catch (Exception e) {
			e.printStackTrace();
			errorlogger.error("[数据同步SEND-THREAD]...异常..."+ this.jobName + "_" + this.threadid + ":" +this.jobBean.getTopic()+","+e.getClass().getName()+":"+e.getMessage(),e);
			logger.error("[数据同步SEND-THREAD]...异常..."+ this.jobName + "_" + this.threadid + ":" +this.jobBean.getTopic()+","+e.getClass().getName()+":"+e.getMessage(),e);
		}

//		ShareVarUtil.setJobSendThreadNum(this.jobName, ShareVarUtil.getJobSendThreadNum(this.jobName)-1);
		ShareVarUtil.addJobSendThreadNum(this.jobName, -1);
		logger.error("[数据同步SEND-THREAD]...End thread..."+ this.jobName + "_" + this.threadid + ":" +this.jobBean.getTopic()+"（"+this.threadid+"）");
	}


	public void cpDoProcess(OrderSyncKafkaBean orderBean, DataSyncJobBean jobBean,boolean flag) {
		logger.info(orderBean.getSyncid()+","+orderBean.getUserid()+","+orderBean.getSynctype()+","+orderBean.getCpid());

		OrderSyncKafkaBeanResult errorOrderBean = null;

		//上次同步时间
		long syncTimestamp = orderBean.getSyncTimestamp();
		if (syncTimestamp == 0) {
			syncTimestamp = System.currentTimeMillis();
			orderBean.setSyncTimestamp(syncTimestamp);
		}

		if (!"3".equals(orderBean.getDatatype())) {
			/**
			 * 线上线下数据处理
			 */
			errorOrderBean = cpSendData(orderBean, jobBean);
		} else {
			/**
			 * 重试数据
			 */
			errorOrderBean = cpSendData(orderBean, jobBean);

//			/**
//			 * 失败重试数据处理
//			 */
//			long interval = (System.currentTimeMillis() - syncTimestamp) / 1000;
//			if (interval >= orderBean.getSynctimes() * jobBean.getDuration()) {
//				errorOrderBean = cpSendData(orderBean, jobBean);
//			} else {
//				/**************************************
//				 * 重试数据，时间间隔没有达到设置值，重新放入到kafka
//				 **************************************
//				 */
//				CommonUtil.sleep((jobBean.getDuration() - interval)*1000L,logger);
//				return;
//			}
		}

		//同步结果处理
		if (!"0".equals(errorOrderBean.getErrorCode())) {
			/************20211102zhengrong.yan，重试不走KAFKA
			if (errorOrderBean.getSynctimes() < 5) {
				orderBean.setDatatype("3");//设置为重试数据
				//还原CPID，用于更新数据和数据重试
				//restoreCpid(orderBean);
				//还原同步前替换的参数，用于重试
				restoreSyncParams(orderBean);
				CommonUtil.sleep(1000,logger);

				/**************************************
				 * 未超过5次重试数据，重新放入到kafka
				 **************************************
				 */
			/****
				boolean fkafka = false;
				try {
					KafkaProductor productor = (KafkaProductor)SpringContextUtils.getBean(KafkaProductor.class);
					fkafka = productor.send(jobBean.getTopic(), jobBean.getTopic(), ConfigUtil.getPartition(orderBean.getUserid(), 3), JSON.toJSONString(orderBean),logger);
					logger.info(jobBean.getTopic()+"，重试推送到KAFKA成功=>"+JSON.toJSONString(orderBean));
				} catch (Exception e) {
					logger.error(jobBean.getTopic()+"，重试推送到KAFKA出现异常=>"+JSON.toJSONString(orderBean),e);
					errorlogger.info(e.getClass().getName()+":"+e.getMessage());
				}

				//放入kafka重试失败，当作失败处理
				if(!fkafka) {
					//更新TD_DATA_SYNC_WAIT表同步结果
					updateSyncLog(orderBean, "2","", false);
					/***
					try {
						dao.updateSyncLog(orderBean.getSyncid(), orderBean.getUserid(), orderBean.getSynctype(), orderBean.getCpid(),"2","");
					} catch (Exception e) {
						CommonUtil.sleep(100L,logger);
						try {
							dao.updateSyncLog(orderBean.getSyncid(), orderBean.getUserid(), orderBean.getSynctype(), orderBean.getCpid(),"2","");
						} catch (Exception e2) {
							errorlogger.error("UPDATE TD_DATA_SYNC_WAIT 失败",e);
							cdnLogger.info("UPDATE TD_DATA_SYNC_WAIT 失败 ,"+e.getClass().getName()+":"+e.getMessage());
						}
					}

				}

			} else {
				/**************************************
				 * 5次重试数据，更新同步结果到标记表
				 **************************************
				 */
				//更新TD_DATA_SYNC_WAIT表同步结果
				if(jobBean.getTopic().contains("fakeids")){
					updateFakeidsSyncLog(orderBean, "2", "", false);
				}else {
					updateSyncLog(orderBean, "2", "", false);
				}/***
				try {
					dao.updateSyncLog(orderBean.getSyncid(), orderBean.getUserid(), orderBean.getSynctype(), orderBean.getCpid(),"2","");
				} catch (Exception e) {
					CommonUtil.sleep(100L,logger);
					try {
						dao.updateSyncLog(orderBean.getSyncid(), orderBean.getUserid(), orderBean.getSynctype(), orderBean.getCpid(),"2","");
					} catch (Exception e2) {
						errorlogger.error("UPDATE TD_DATA_SYNC_WAIT 失败",e);
						cdnLogger.info("UPDATE TD_DATA_SYNC_WAIT 失败 ,"+e.getClass().getName()+":"+e.getMessage());
					}
				}

			}
				 ***/
		} else {
			if(flag) {
				//同步成功
				//更新TD_DATA_SYNC_WAIT表同步结果
				if(jobBean.getTopic().contains("fakeids")){
					updateFakeidsSyncLog(orderBean, "0", "", false);
				}else {
					updateSyncLog(orderBean, "0", "", false);
				}/***
				try {
					dao.updateSyncLog(orderBean.getSyncid(), orderBean.getUserid(), orderBean.getSynctype(), orderBean.getCpid(),"0","");
				} catch (Exception e) {
					CommonUtil.sleep(100L,logger);
					try {
						dao.updateSyncLog(orderBean.getSyncid(), orderBean.getUserid(), orderBean.getSynctype(), orderBean.getCpid(),"0","");
					} catch (Exception e2) {
						errorlogger.error("UPDATE TD_DATA_SYNC_WAIT 失败",e);
						cdnLogger.info("UPDATE TD_DATA_SYNC_WAIT 失败 ,"+e.getClass().getName()+":"+e.getMessage());
					}
				}
				***/
			}
		}

		// 往日志文件写入记录 以及 往数据库写入 记录
		String logStr = JSON.toJSONString(errorOrderBean);
		if ("0".equals(errorOrderBean.getErrorCode()) || "0000".equals(errorOrderBean.getErrorCode())) {
			logger.info(logStr);
		} else {
			logger.info(logStr);
		}


		//同步日志中，记录同步的地址
		try {
			String output = errorOrderBean.getOutput();
			if(output==null){
				output="";
			}
			if("fengx".equals(errorOrderBean.getCpid())){
				output = output+",URL="+errorOrderBean.getUrl();
			}else{
				output = output+",URL="+jobBean.getUrl();
			}
			errorOrderBean.setOutput(output);
		} catch (Exception e) {
			e.printStackTrace();
		}

		try {
			dao.insertCpSyncSendLog(errorOrderBean);
		} catch (Exception e) {
			logger.error(jobBean.getTopic()+"，添加同步日志出现异常"+orderBean.getSyncid()+","+orderBean.getUserid()+","+orderBean.getSynctype()+","+orderBean.getCpid(),e);
			errorlogger.error("添加CP同步日志出现异常，"+e.getMessage(),e);
			//throw e;
		}

	}

	/**
	 * 向cp发起数据同步请求
	 *
	 */
	public OrderSyncKafkaBeanResult cpSendData(OrderSyncKafkaBean orderBean, DataSyncJobBean jobBean) {
		//同步次数
		int synctimes = orderBean.getSynctimes();
		orderBean.setSynctimes(synctimes + 1);

		// 同步结果 的数据初始化
		OrderSyncKafkaBeanResult errorOrderInfo = cpSyncResult(orderBean);

		// 一般cp同步数据 的格式
		OrderCpSyncBean syncBean = cpOrderSyncBean(orderBean);

		// 优酷同步数据的格式
		OrderCpSyncBeanYoutu syncBeanYoutu = cpOrderSyncBeanYoutu(orderBean);

		/**
		 * 风行的同步特别处理
		 */
		String url = jobBean.getUrl();
		if("fengx".equals(orderBean.getCpid())){
			url = url+(new Date().getTime()/1000L);
		}
		errorOrderInfo.setUrl(url);

		/**
		 * 优酷同步特殊处理
		 */
		String data = JSON.toJSONString(syncBean);
		if ("youtu".equals(orderBean.getCpid())) {
			data = JSON.toJSONString(syncBeanYoutu);

//			String fakeid = syncBeanYoutu.getFakeid();
//			try {
//				fakeid = URLDecoder.decode(fakeid, "utf-8");
//			} catch (Exception e) {}
//
//			int i1 = data.indexOf("\"fakeid\":\"");
//			String temp1 = data.substring(0,i1+10);
//			String temp2 = data.substring(i1+10);
//			int i2 = temp2.indexOf("\"");
//			data = temp1 +fakeid+ temp2.substring(i2);

		/**
		 * 快手同步特殊处理
		 */
		}else if ("kuaishou".equals(orderBean.getCpid())) {
			String signdata = syncBean.getUsermob()+syncBean.getCpid()+syncBean.getSpid()+"2rFUrLZyKV9S";
			String signstr = MD5.MD5Encode(signdata);
			String sign = signstr.length()>16?signstr.substring(0,16):signstr;

			data = data.replace("{", "{\"sign\":\""+sign+"\",");
		/**
		 * 葫芦侠另外处理，接口参数与返回值与通用的不一样
		 */
		}else if ("hlx".equals(orderBean.getCpid())){
			try {
				List<HashMap> realIds = dao.getRealIds(orderBean.getUserid(), orderBean.getCpid(), orderBean.getSpid());
				String orderid = realIds.get(0).get("ID").toString();
				String timestamp = (System.currentTimeMillis())/1000+"";
				String pwd = "hlx44e50";
				String channel = orderBean.getChannelcode();
				String type = orderBean.getType();
				String eduserid = DesUtil.encode(orderBean.getUserid(), pwd);
				String canceltime = orderBean.getCanceltime();
				String ordertime = orderBean.getOrdertime();
				String endtime = orderBean.getEndtime();
				String cpid = orderBean.getCpid();
				String spid = orderBean.getSpid();
				String signature = orderid+cpid+spid+eduserid+type+channel+timestamp+pwd;
				signature = MD5.MD5Encode(signature);

				JSONObject params = new JSONObject();
				params.put("orderid", orderid);
				params.put("type", type);
				params.put("userid", eduserid);
				params.put("channel", channel);
				params.put("cpid", cpid);
				params.put("spid", spid);
				params.put("timestamp", timestamp);
				params.put("signature", signature);
				params.put("canceltime", canceltime);
				params.put("ordertime", ordertime);
				params.put("endtime", endtime);

				data = params.toJSONString();
			} catch (Exception e) {
				e.printStackTrace();
				logger.error("葫芦侠CP同步组建参数出现异常，"+e.getMessage()+"，data："+data);
				errorOrderInfo.setInput(data);
				errorOrderInfo.setErrorCode("1");
				errorOrderInfo.setErrorInfo("组建参数出现异常，"+e.getMessage());
				if (errorOrderInfo.getErrorInfo() != null && errorOrderInfo.getErrorInfo().length() > 200) {
					errorOrderInfo.setErrorInfo(errorOrderInfo.getErrorInfo().substring(0, 200));
				}
				errorOrderInfo.setCostTime(0);
				errorOrderInfo.setOutput(errorOrderInfo.getErrorInfo());
				return errorOrderInfo;
			}
		}
		errorOrderInfo.setInput(data);

		long costTime = System.currentTimeMillis();
		try {
			/**
			 * 调用CP同步接口
			 */
			//String reqResult = "";

			//同步次数
			int retrycount = 0;
			//最大重试次数
			int maxretrycount = 4;
			//调接口返回值
			String reqResult = null;
			//是否调接口
			boolean hasinvoke = true;
			//调CDN同步接口，失败则重试
			while(hasinvoke) {
				reqResult = null;
				retrycount++;
				//未达到最大重试次数
				if(retrycount <= maxretrycount){
					if(retrycount > 1){
						//设置同步次数
						orderBean.setSynctimes(orderBean.getSynctimes() + 1);
					}
					logger.info("CP第"+retrycount+"次同步，"+data);
					try {
						//调接口
						invokeCPSync(retrycount);//是否需要休眠
						if("xybd".equals(orderBean.getCpid())){
							reqResult = URLUtil.post(url, data, 10000,"application/json");
						}else if("xueguoxue".equals(orderBean.getCpid())) {
							System.setProperty("https.protocols", "TLSv1.2,TLSv1.1,TLSv1,SSLv3");
							reqResult = URLUtil.post(url, data, 10000,"application/json");
						}else if("zmkb".equals(orderBean.getCpid())) {
							reqResult = URLUtil.post(url, data, 10000,"application/json");
						}else if("kuaishou".equals(orderBean.getCpid())) {
							reqResult = URLUtil.post(url, data, 10000,"application/json");
						}else if("wotv".equals(orderBean.getCpid())){
							//System.setProperty("https.protocols", "TLSv1.2,TLSv1.1,TLSv1,SSLv3");
							//reqResult = URLUtil.post(url, data, 10000);
							String method = "POST";
							String tls = "TLSv1.2";
							//reqResult = URLUtil.postByTls(url, data, 10000, "text/plain", "TLSv1.2");
							Map reqProperty = new HashMap();
							reqProperty.put("content-type", "text/plain");
							//reqResult = HttpInvoke.sendhttpsReq("POST", url, data, reqProperty, 10, "TLSv1.2");
							reqResult = HttpInvoke.sendhttpsReqAll(method, url, data, reqProperty, 10*1000, tls);
						} else if("keep".equals(orderBean.getCpid())){
							//System.setProperty("https.protocols", "TLSv1.2,TLSv1.1,TLSv1,SSLv3");
							//reqResult = URLUtil.post(url, data, 10000);
							String method = "POST";
							String tls = "TLSv1.2";
							//reqResult = URLUtil.postByTls(url, data, 10000, "text/plain", "TLSv1.2");
							Map reqProperty = new HashMap();
							reqProperty.put("content-type", "application/json");
							//reqResult = HttpInvoke.sendhttpsReq("POST", url, data, reqProperty, 10, "TLSv1.2");
							reqResult = HttpInvoke.sendhttpsReqAll(method, url, data, reqProperty, 10*1000, tls);
						}else{
							reqResult = URLUtil.post(url, data, 10000);
						}
						//同步成功
						if(reqResult != null
								&& JSONObject.parseObject(reqResult) != null
								&& ("0".equals(JSONObject.parseObject(reqResult).getString("result"))
								|| ("0".equals(JSONObject.parseObject(reqResult).getString("resultcode"))))){//hlx 结果是resultcode
							hasinvoke = false;
						}
					}catch (Exception e){
						if(retrycount == maxretrycount){
							errorOrderInfo.setSynctimes(orderBean.getSynctimes());
							throw e;
						}
					}
				}else{
					hasinvoke = false; //重试超过最大次数，跳出循环
				}
			}
			errorOrderInfo.setSynctimes(orderBean.getSynctimes());
//			if("xybd".equals(orderBean.getCpid())){
//				reqResult = URLUtil.post(url, data, 10000,"application/json");
//			}else if("xueguoxue".equals(orderBean.getCpid())) {
//				System.setProperty("https.protocols", "TLSv1.2,TLSv1.1,TLSv1,SSLv3");
//				reqResult = URLUtil.post(url, data, 10000,"application/json");
//			}else if("zmkb".equals(orderBean.getCpid())) {
//				reqResult = URLUtil.post(url, data, 10000,"application/json");
//			}else if("kuaishou".equals(orderBean.getCpid())) {
//				reqResult = URLUtil.post(url, data, 10000,"application/json");
//			}else{
//				reqResult = URLUtil.post(url, data, 10000);
//			}

			/**
			 * 调用CP同步接口 结果处理
			 */
			if (reqResult != null) {
				errorOrderInfo.setErrorInfo(reqResult);
				//葫芦侠返回格式不一样，单独处理{\"resultcode\":\"0\",\"errorInfo\":\"\"}
				if ("hlx".equals(orderBean.getCpid())){
					String resultcode = "";
		            if (reqResult != null && !"".equals(reqResult)) {
		            	JSONObject resultObject = JSONObject.parseObject(reqResult);
		                resultcode = resultObject.getString("resultcode");
		            }else{
		            	resultcode = "5000";
		            }
		            errorOrderInfo.setErrorCode(resultcode);
				}else{
					JSONObject resultObject = JSONObject.parseObject(reqResult);
					String result = resultObject.getString("result");
					errorOrderInfo.setErrorCode(result);
				}
			} else {
				errorOrderInfo.setErrorCode("1");
				errorOrderInfo.setErrorInfo("接口返回异常，无响应结果");
			}
		} catch (Exception e) {
			/**
			 * 调用CP同步接口 异常
			 */
			errorOrderInfo.setErrorCode("1");
			String temp = (errorOrderInfo.getErrorInfo()==null?"":(errorOrderInfo.getErrorInfo()+",")) + e.getClass().getName()+": "+e.getMessage().toString();
			if (temp != null && temp.length() > 200) {
				errorOrderInfo.setErrorInfo(temp.substring(0, 200));
			} else {
				errorOrderInfo.setErrorInfo(temp);
			}
		} finally {
			costTime = System.currentTimeMillis() - costTime;
			errorOrderInfo.setCostTime((int) costTime);
			errorOrderInfo.setOutput(errorOrderInfo.getErrorInfo());
		}
		return errorOrderInfo;
	}

	/**
	 * 调CP同步接口休眠
	 * @param url 接口地址
	 * @param data 数据
	 * @param timeout 超时时间
	 * @param retrycount 重试次数
	 * @return
	 * @throws Exception
	 */
	private void invokeCPSync(int retrycount) throws Exception {
		//重试要休眠一会
		if(retrycount > 1){
			CommonUtil.sleep(500,logger);
		}
	}

	/**
	 * 常规cp同步数据
	 *
	 * @param orderBean
	 * @return
	 */
	public OrderCpSyncBean cpOrderSyncBean(OrderSyncKafkaBean orderBean) {
		OrderCpSyncBean syncBean = new OrderCpSyncBean();
		syncBean.setArea(orderBean.getArea());
		syncBean.setCanceltime(orderBean.getCanceltime());
		syncBean.setChannelcode(orderBean.getChannelcode());
		syncBean.setCpid(orderBean.getCpid());
		syncBean.setEndtime(orderBean.getEndtime());
		syncBean.setOrdertime(orderBean.getOrdertime());
		syncBean.setOrdertype(orderBean.getOrdertype());
		syncBean.setProvince(orderBean.getProvince());
		syncBean.setSpid(orderBean.getSpid());
		syncBean.setType(orderBean.getType());
		syncBean.setUsermob(orderBean.getUsermob());
		syncBean.setVideoid(orderBean.getVideoid());

		if("aiqy".equals(orderBean.getCpid()) || "kuaishou".equals(orderBean.getCpid())) {
			try {
				syncBean.setFakeid(URLEncoder.encode(orderBean.getFakeid(), "utf-8"));
			} catch (Exception e) {}
		}else if ("bilibl".equals(orderBean.getCpid())){
			syncBean.setFakeid(orderBean.getFakeid());
			syncBean.setFakeidmonth((new SimpleDateFormat("YYYYMM").format(new Date())));
		} else {
			syncBean.setFakeid(orderBean.getFakeid());
		}


		if("2".equals(syncBean.getType())) {
			syncBean.setType("1");
		}

		return syncBean;
	}


	/**
	 * 优酷cp同步数据
	 *
	 * @param orderBean
	 * @return
	 */
	public OrderCpSyncBeanYoutu cpOrderSyncBeanYoutu(OrderSyncKafkaBean orderBean) {
		OrderCpSyncBeanYoutu syncBean = new OrderCpSyncBeanYoutu();
		syncBean.setCanceltime(orderBean.getCanceltime());
		syncBean.setChannelcode(orderBean.getChannelcode());
		syncBean.setCpid(orderBean.getCpid());
		syncBean.setEndtime(orderBean.getEndtime());
		syncBean.setOrdertime(orderBean.getOrdertime());
		syncBean.setOrdertype(orderBean.getOrdertype());
		syncBean.setSpid(orderBean.getSpid());
		syncBean.setType(orderBean.getType());
		syncBean.setUsermob(orderBean.getUsermob());
		syncBean.setFakeid(orderBean.getFakeid());

		if("2".equals(syncBean.getType())) {
			syncBean.setType("1");
		}

		return syncBean;
	}

	public OrderSyncKafkaBeanResult cpSyncResult(OrderSyncKafkaBean syncOrderInfoBean) {
		OrderSyncKafkaBeanResult errorOrderInfo = new OrderSyncKafkaBeanResult();
		if (syncOrderInfoBean == null) {
			return errorOrderInfo;
		}

		errorOrderInfo.setArea(syncOrderInfoBean.getArea());
		errorOrderInfo.setCanceltime(syncOrderInfoBean.getCanceltime());
		errorOrderInfo.setChannelcode(syncOrderInfoBean.getChannelcode());
		errorOrderInfo.setCostTime(syncOrderInfoBean.getCostTime());
		errorOrderInfo.setCpid(syncOrderInfoBean.getCpid());
		errorOrderInfo.setEndtime(syncOrderInfoBean.getEndtime());
		errorOrderInfo.setOrdertime(syncOrderInfoBean.getOrdertime());
		errorOrderInfo.setOrdertype(syncOrderInfoBean.getOrdertype());
		errorOrderInfo.setFromCpid(syncOrderInfoBean.getFromCpid());
		errorOrderInfo.setFromSpid(syncOrderInfoBean.getFromSpid());
		errorOrderInfo.setProvince(syncOrderInfoBean.getProvince());
		errorOrderInfo.setSpid(syncOrderInfoBean.getSpid());
		errorOrderInfo.setSynctimes(syncOrderInfoBean.getSynctimes());
		errorOrderInfo.setSyncTimestamp(syncOrderInfoBean.getSyncTimestamp());
		errorOrderInfo.setType(syncOrderInfoBean.getType());
		errorOrderInfo.setUserid(syncOrderInfoBean.getUserid());
		errorOrderInfo.setUsermob(syncOrderInfoBean.getUsermob());
		errorOrderInfo.setValid(syncOrderInfoBean.getValid());
		errorOrderInfo.setVideoid(syncOrderInfoBean.getVideoid());
		errorOrderInfo.setDatatype(syncOrderInfoBean.getDatatype());

		// errorOrderInfo.setErrorCode(syncOrderInfoBean.gete);
		// errorOrderInfo.setErrorInfo(syncOrderInfoBean.getCanceltime());
		// errorOrderInfo.setInput(syncOrderInfoBean);
		// errorOrderInfo.setInserttime(syncOrderInfoBean);
		// errorOrderInfo.setOutput(syncOrderInfoBean.getCanceltime());

		if("2".equals(errorOrderInfo.getType())) {
			errorOrderInfo.setType("1");
		}

		return errorOrderInfo;
	}

	/**
	 * cdn同步处理
	 *
	 * @param orderBean
	 * @param jobBean
	 */
	public void cdnDoProcess(OrderSyncKafkaBean orderBean, DataSyncJobBean jobBean) {
		long starttime = System.currentTimeMillis();

		//同步次数+1
		orderBean.setSynctimes(orderBean.getSynctimes() + 1);

		// 生成 同步给CDN的订购关系数据 bean
//		OrderCdnSyncBean syncBean = cdnOrderSyncBean(orderBean,key);
//		String data = JSON.toJSONString(syncBean);

		//向cdn同步数据
//		OrderSyncKafkaBeanResult errorOrderBean = cdnSendData(url, data, orderBean ,timeout);
//		errorOrderBean.setTime((int) (System.currentTimeMillis() - starttime));

		//同步处理结果
//		if (!"0".equals(errorOrderBean.getErrorCode())) {
			/************20211101xu.zhou，重试不走KAFKA
			if (errorOrderBean.getSynctimes() < 5) {
				//还原同步前替换的参数，用于重试
				restoreSyncParams(orderBean);
				CommonUtil.sleep(1000,logger);

				/**************************************
				 * 未超过5次重试数据，重新放入到kafka
				 **************************************
				 */
				/*****
				boolean fkafka = false;
				try {
					KafkaProductor productor = (KafkaProductor)SpringContextUtils.getBean(KafkaProductor.class);
					fkafka = productor.send(jobBean.getTopic(), jobBean.getTopic(), ConfigUtil.getPartition(orderBean.getUserid(), 3), JSON.toJSONString(orderBean),cdnLogger);
				} catch (Exception e) {
					errorlogger.info(e.getClass().getName()+":"+e.getMessage());
				}

				//放入kafka重试失败，当作失败处理
				if(!fkafka) {
					//更新TD_DATA_SYNC_WAIT表同步结果
					updateSyncLog(orderBean, "2","", true);
				}
			}else {
			********/
				//同步失败
				//更新TD_DATA_SYNC_WAIT表同步结果
//				updateSyncLog(orderBean, "2","", true);
			//}
//		} else {
			//同步成功
			//更新TD_DATA_SYNC_WAIT表同步结果
			updateSyncLog(orderBean, "0","", true);
//		}

		//记录同步日志
//		cdnLogger.info(JSON.toJSONString(errorOrderBean));
//		try {
//			dao.insertCdnSyncSendLog(errorOrderBean);
//		} catch (Exception e) {
//			cdnLogger.error(jobBean.getTopic()+"，添加同步日志出现异常"+orderBean.getSyncid()+","+orderBean.getUserid()+","+orderBean.getSynctype()+","+orderBean.getCpid(),e);
//			errorlogger.error("添加CDN同步日志出现异常，"+e.getMessage(),e);
//			//throw e;
//		}

	}

	/**
	 * 同步cdn
	 *
	 * @param url
	 * @param data
	 * @param orderBean
	 * @param timeout
	 * @return
	 */
	private OrderSyncKafkaBeanResult cdnSendData(String url, String data, OrderSyncKafkaBean orderBean, int timeout) {
		OrderSyncKafkaBeanResult errorInfo = cpSyncResult(orderBean);
		errorInfo.setType(orderBean.getOrdertype());

		if("2".equals(errorInfo.getType())) {
			errorInfo.setType("1");
		}

		errorInfo.setInput(data);
		try {
			//同步次数
			int retrycount = 0;
			//最大重试次数
			int maxretrycount = 4;
			//调接口返回值
			String reqResult = null;
			//是否调接口
			boolean hasinvoke = true;
			//调CDN同步接口，失败则重试
			while(hasinvoke){
				reqResult = null;
				retrycount++;
				//未达到最大重试次数
				if(retrycount <= maxretrycount){
					if(retrycount > 1){
						//设置同步次数
						orderBean.setSynctimes(orderBean.getSynctimes() + 1);
					}
					cdnLogger.info("CDN第"+retrycount+"次同步，"+data);
					try {
						reqResult = invokeCdnSync(url, data, timeout, retrycount);
						//同步成功
						if(reqResult != null
								&& JSONObject.parseObject(reqResult) != null
								&& "0".equals(JSONObject.parseObject(reqResult).getString("result"))){
							hasinvoke = false;
						}
					} catch (Exception e) {
						if(retrycount == maxretrycount){
							errorInfo.setSynctimes(orderBean.getSynctimes());
							throw e;
						}
					}
				}else{
					hasinvoke = false; //重试超过最大次数，跳出循环
				}
			}

			errorInfo.setSynctimes(orderBean.getSynctimes());

			//String reqResult = URLUtil.post( url, data, timeout);

			/**
			 * 调用CP同步接口 结果处理
			 */
			if (reqResult != null) {
				errorInfo.setErrorInfo(reqResult);
				JSONObject resultObject = JSONObject.parseObject(reqResult);
				String result = resultObject.getString("result");
				errorInfo.setErrorCode(result);

			} else {
				errorInfo.setErrorCode("9000");
				errorInfo.setErrorInfo("接口返回异常，无响应结果");
			}
		} catch (Exception e) {
			errorInfo.setErrorCode("9000");
			String errorInfotmp = e.getMessage().toString();
			if (errorInfotmp.length() > 100) {
				errorInfo.setErrorInfo(e.getMessage().toString().substring(0, 100));
			} else {
				errorInfo.setErrorInfo(e.getMessage().toString());
			}
		} finally {
			errorInfo.setOutput(errorInfo.getErrorInfo());
		}
		return errorInfo;
	}

	/**
	 * 调CDN同步接口
	 * @param url 接口地址
	 * @param data 数据
	 * @param timeout 超时时间
	 * @param retrycount 重试次数
	 * @return
	 * @throws Exception
	 */
	private String invokeCdnSync(String url, String data, int timeout, int retrycount) throws Exception {
		//重试要休眠一会
		if(retrycount > 1){
			CommonUtil.sleep(500,logger);
		}
		return URLUtil.post( url, data, timeout);
	}

	/**
	 * 同步给cdn的数据
	 *
	 * @param orderBean
	 * @param key
	 * @return
	 */
	private OrderCdnSyncBean cdnOrderSyncBean(OrderSyncKafkaBean orderBean,String key) {
		OrderCdnSyncBean syncBean = new OrderCdnSyncBean();
		syncBean.setArea(orderBean.getArea());
		syncBean.setCanceltime(orderBean.getCanceltime());
		syncBean.setChannelcode(orderBean.getChannelcode());
		syncBean.setCpid(orderBean.getCpid());
		syncBean.setEndtime(orderBean.getEndtime());
		syncBean.setOrdertime(orderBean.getOrdertime());
		syncBean.setProvince(orderBean.getProvince());
		syncBean.setSpid(orderBean.getSpid());
		syncBean.setUsermob(orderBean.getUserid());
		syncBean.setVideoid(orderBean.getVideoid());
		syncBean.setType(orderBean.getType());
		syncBean.setOrdertype(orderBean.getOrdertype());

		if("2".equals(syncBean.getOrdertype())) {
			syncBean.setOrdertype("1");
		}

//		syncBean.setType(orderBean.getOrdertype());
//		syncBean.setOrdertype(orderBean.getType());
//		syncBean.setFakeid(orderBean.getFakeid());

		String time = new SimpleDateFormat("yyyyMMddHHmmss").format( new Date() );
		String mkey = MD5.MD5Encode(time + key);
		syncBean.setMkey(mkey);
		syncBean.setTime(time);

		return syncBean;
	}

	public static void main(String[] args) {
		Map reqProperty = new HashMap();
		reqProperty.put("content-type", "text/plain");
		String data = "{\"area\":\"长沙\",\"canceltime\":\"20220810110034\",\"channelcode\":\"kc\",\"cpid\":\"wotv\",\"endtime\":\"20220831235959\",\"ordertime\":\"20220810103058\",\"ordertype\":\"3\",\"province\":\"湖南\",\"spid\":\"1113\",\"type\":\"1\",\"usermob\":\"DSm0mBohWaxNSyon8GP4Hw%3D%3D\",\"videoid\":\"\"}";
		String url = "https://woopen.chinaunicomvideo.cn/wovideo/3.0.0/kdgs/orderCallback";
		//url = "https://www.baidu.com";
		String reqResult = "";
		String method = "POST";
		String tls = "TLSv1.2";
		try {
			reqResult = HttpInvoke.sendhttpsReqAll(method, url, data, reqProperty, 10*1000, tls);
			//reqResult =  HttpInvoke.sendhttpsReq(method, url, data, reqProperty, 10*1000, tls);
			//reqResult = URLUtil.postByTls(url, data, 10000, "text/plain", "TLSv1.2");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println(reqResult);
	}
}
