package com.chinacreator.datasync.kafka;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Future;

import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.Producer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.clients.producer.RecordMetadata;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.chinacreator.datasync.common.CommonUtil;

import kafka.serializer.StringEncoder;

/**
 * 
 * 
 * 
 * @author maoya
 *
 */
@Component
public class KafkaProductor {
	
	public static Logger logger = LoggerFactory.getLogger("KAFKA");
	
    @Value("${config.kafka.producer.zk.connect}")
    private String zk_connect;
    @Value("${config.kafka.producer.bootstrap.servers}")
    private String bootstrap_servers;
    @Value("${config.kafka.producer.metadata.broker.list}")
    private String metadata_broker_list;
	
	private Producer<String, String> producer;
	
	protected void finalize() {
		if (producer != null) {
			logger.info("producer close ... .... ... ");
			producer.close();
		}
	}
	
	public void close() {
		if (producer != null) {
			producer.close();
		}
	}
	
    public Map<String, Object> initKafkaConfig() {
        Map<String, Object> props = new HashMap<>();
        props.put("zk.connect", zk_connect);
        props.put("bootstrap.servers", bootstrap_servers);
        props.put("metadata.broker.list", metadata_broker_list);
        props.put("buffer.memory", 67108864);
        props.put("retries", 3);
        props.put("acks", "-1");
        props.put("serializer.class", StringEncoder.class.getName());
        props.put("key.serializer", "org.apache.kafka.common.serialization.StringSerializer");
        props.put("value.serializer", "org.apache.kafka.common.serialization.StringSerializer");
        return props;
    }

	public void initKafka() {
		try {
			producer = new KafkaProducer<>(initKafkaConfig());
			logger.info("初始化重试kafka-producer:成功");
		} catch (Exception e) {
			e.printStackTrace();
			logger.info("初始化重试kafka-producer:异常，"+e.getClass().getName()+":"+e.getMessage());
		}
	}

	
	public boolean send(String topic, String groupid, int partitionNum, String value,Logger log) {
		try {
			if(producer == null) {
				initKafka();
			}
			
			ProducerRecord<String, String> record = null;
			if (CommonUtil.isNotBlank(groupid)) {
				record = new ProducerRecord<>(topic, partitionNum, groupid,value);
			} else {
				record = new ProducerRecord<>(topic, partitionNum, null, value);
			}
			Future<RecordMetadata> result = producer.send(record);
			if(result!=null) {
				result.get();
			}
			log.info("发送kafka重试:"+topic+"成功");
			
			return true;
		} catch (Exception e) {
			e.printStackTrace();
			log.info("发送kafka重试:"+topic+"异常，"+e.getClass().getName()+":"+e.getMessage());
		}
		
		return false;
	}

}
