package com.chinacreator.datasync.services;

import com.chinacreator.datasync.common.ShareVarUtil;
import com.chinacreator.datasync.dao.QueryDbDao;
import com.chinacreator.datasync.entity.DataSyncJobBean;
import com.chinacreator.datasync.entity.JobStatusInfoBean;
import com.chinacreator.datasync.entity.OrderSyncKafkaBean;
import com.chinacreator.datasync.handler.DataSyncOrderConsumerHandler;
import com.chinacreator.datasync.handler.DataSyncOrderConsumerHandlerNew;
import com.chinacreator.datasync.handler.DataSyncOrderSendHandler;
import com.chinacreator.datasync.util.ParamsConfig;
import com.chinacreator.datasync.util.SpringContextUtils;

import org.apache.log4j.Logger;
import org.quartz.*;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 *
 * @author zhengrong.yan
 *
 */
// 保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
// 保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class DataSyncOrderJobNew implements Job {
	
	private static Logger logger = Logger.getLogger("syncnew");
	
	public static int consumer_coreThrnum = 2000;
	public static int consumer_maxThrnum = 2000;
	
	public static int send_coreThrnum = 2100;
	public static int send_maxThrnum = 2100;
	
	public static Map<String, BlockingQueue<OrderSyncKafkaBean>> syncOrderQueueMap = new HashMap<String, BlockingQueue<OrderSyncKafkaBean>>();

	@Override
	public void execute(JobExecutionContext context) throws JobExecutionException {
    	//每次取数据的条数
    	int maxpollrecords = 50;
		
		//#CDN同步内存队列长度设置
		int queuelencdn = 100;
		
		//#CP同步内存队列长度设置
		int queuelencp = 50;
		
		String autooffsetreset = ParamsConfig.autooffsetreset;
		
		try {
			queuelencdn = Integer.parseInt(ParamsConfig.queuelencdn);
		} catch (Exception e) {
			queuelencdn = 100;
		}
		
		try {
			queuelencp = Integer.parseInt(ParamsConfig.queuelencp);
		} catch (Exception e) {
			queuelencp = 50;
		}
		
    	try {
    		maxpollrecords = Integer.parseInt(ParamsConfig.maxpollrecords);
		} catch (Exception e) {
			maxpollrecords = 50;
		}
		
		logger.info("queuelencdn=>"+queuelencdn+"，queuelencp=>"+queuelencp);
		
		DataSyncJobBean syncJob = (DataSyncJobBean)context.getJobDetail().getJobDataMap().get("data");
		
		QueryDbDao dao = (QueryDbDao) SpringContextUtils.getBean(QueryDbDao.class);
		
		if(syncOrderQueueMap.get(syncJob.getName()) == null) {
			BlockingQueue<OrderSyncKafkaBean> syncOrderQueue = null;
			if("cdn".equals(syncJob.getCpid())) {
				syncOrderQueue = new LinkedBlockingQueue<OrderSyncKafkaBean>(queuelencdn);
			}else {
				syncOrderQueue = new LinkedBlockingQueue<OrderSyncKafkaBean>(queuelencp);
			}
			syncOrderQueueMap.put(syncJob.getName(), syncOrderQueue);
		}
		
		////////// 初始化数据队列 ///////////
		///////// job 运行状态信息 获取判断
		///流程   cdn统一进行处理 数据到内存队列，consumer取数据到内存队列--->真正开始消费（此处不需要进行修改
		//放入内存队列
		JobStatusInfoBean jobStatusInfoBean = ShareVarUtil.getJobStatusInfo(syncJob.getName());
		if (jobStatusInfoBean == null) {
			jobStatusInfoBean = new JobStatusInfoBean();
			jobStatusInfoBean.setCpid(syncJob.getCpid());
			jobStatusInfoBean.setJobName(syncJob.getName());
			jobStatusInfoBean.setQueueStatus("1");
			jobStatusInfoBean.setStatus(syncJob.getStatus());
			ShareVarUtil.setJobStatusInfo(syncJob.getName(),jobStatusInfoBean);
		}
		
		int consumerThreadNum = ShareVarUtil.getJobConsumerThreadNum(syncJob.getName());
		try {
			/////////////// 当前job开启的 消费线程数据是否与配置所需要的 线程数一致，若一致则不再新建线程 //////
			logger.info(syncJob.getName()+" 当前消费线程数：consumerThreadNum = "+consumerThreadNum+"... ...");
			if (consumerThreadNum < 1) {
				ShareVarUtil.setJobConsumerThreadNum(syncJob.getName(),1);
//				ShareVarUtil.addJobConsumerThreadNum(syncJob.getName(),1);
				DataSyncOrderConsumerHandlerNew handler = new DataSyncOrderConsumerHandlerNew(syncOrderQueueMap.get(syncJob.getName()), syncJob,syncJob.getName(), 1,
						queuelencdn, queuelencp, maxpollrecords, autooffsetreset);
				handler.run();
			}
			/////////////// 当前job开启的 消费线程 end //////////////////////////////////////
		} catch (Exception e) {
			e.printStackTrace();
		}
		

		///kafka开始进行真正的消费
		int sendThreadNum = ShareVarUtil.getJobSendThreadNum(syncJob.getName());
		try {
			/////////////// 当前job开启的 发送线程数据是否与配置所需要的 线程数一致，若一致则不再新建线程
			int sendNum = syncJob.getSendThreadNum();
			logger.info(syncJob.getName()+" 当前发送线程数：sendThreadNum = "+sendThreadNum+"..."+sendNum);
			if (sendThreadNum < sendNum) {
				ThreadPoolExecutor kafakSendExetor = new ThreadPoolExecutor(send_coreThrnum, send_maxThrnum, 0, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<Runnable>(5));
				
				for (int i = sendThreadNum; i < sendNum; i++) {
//					ShareVarUtil.setJobSendThreadNum(syncJob.getName(),ShareVarUtil.getJobSendThreadNum(syncJob.getName())+1);
					ShareVarUtil.addJobSendThreadNum(syncJob.getName(),1);
					
					sendThreadNum = sendThreadNum+1;
					kafakSendExetor.execute(new DataSyncOrderSendHandler(syncOrderQueueMap.get(syncJob.getName()), syncJob, syncJob.getName(),sendThreadNum,dao));
				}
			}
			/////////////// 当前job开启的 发送线程 end //////////////////////////////////////
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
}
