package com.chinacreator.datasync.services;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.quartz.CronScheduleBuilder;
import org.quartz.CronTrigger;
import org.quartz.Job;
import org.quartz.JobBuilder;
import org.quartz.JobDataMap;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.SchedulerException;
import org.quartz.TriggerBuilder;
import org.quartz.TriggerKey;
import org.quartz.impl.StdSchedulerFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.chinacreator.datasync.common.ShareVarUtil;
import com.chinacreator.datasync.dao.QueryDbDao;
import com.chinacreator.datasync.entity.DataSyncJobBean;

@Component
public class SyncService {
	
	public static boolean flag = false;
	
	public static String JOB_CLASS_NAME="com.chinacreator.datasync.services.DataSyncOrderJobNew";
	
	public static String JOB_CRON="0 */2 * * * ?";	
	
	@Value("${config.job.sync.dealtype}")
	public String dealtype;

	public Logger logger = LoggerFactory.getLogger("sync");
	
	private org.quartz.Scheduler scheduler;
	
	
	@Autowired
    private QueryDbDao dao; 
	
	public void sync(){
		System.out.println(">>>>>>>>>"+new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
		
		try {
			
			if(scheduler== null) {
				scheduler = StdSchedulerFactory.getDefaultScheduler();
			}
			
			if(!flag) {
				flag = true;
				
				initJob(scheduler);
				start(scheduler);
			}else {
				jobStatusUpdate(scheduler);
				start(scheduler);
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		

	}
	
	/**
	 * 任务初始化
	 * 
	 * @param scheduler
	 * @throws SchedulerException
	 * @throws ClassNotFoundException
	 */
	public void initJob(org.quartz.Scheduler scheduler) throws SchedulerException, ClassNotFoundException {
		if (scheduler != null) {
			logger.info("【任务初始化】 .... ....");
			addCPSyncOrderJob(scheduler);
		}
	}

	/**
	 * 任务初始化 - 添加所有任务
	 * 
	 * @param scheduler
	 * @throws SchedulerException
	 * @throws ClassNotFoundException
	 */
	private void addCPSyncOrderJob(org.quartz.Scheduler scheduler) throws SchedulerException, ClassNotFoundException {
		logger.info("【初始化Job】");
		
		List<DataSyncJobBean> list = null;
		if(dealtype == null || "".equals(dealtype) || "all".equals(dealtype)) {
			list = dao.querySyncJob();
		}else {
			list = dao.querySyncJobByType(dealtype);
		}
		if (list != null && list.size() > 0) {
			for (DataSyncJobBean syncJob : list) {
				syncJob.setCronExpression(JOB_CRON);
				String jobStatus = syncJob.getStatus();
				if (!"0".equals(jobStatus)) {
					continue;
				}
				
				logger.info("【初始化Job】  " + syncJob.getName());
				
				try {
					Class<? extends Job> jobClass = (Class<? extends Job>) Class.forName(JOB_CLASS_NAME);
					addJob(scheduler,syncJob.getName(), syncJob.getCronExpression(), jobClass, syncJob, jobStatus);
				} catch (Exception e) {
					e.printStackTrace();
					logger.info("初始化Job failed .... ....");
				}
			}
		} 
	}
	
	/**
	 * 任务启动
	 * 
	 * @param scheduler
	 * @throws SchedulerException
	 */
	public void start(org.quartz.Scheduler scheduler) throws SchedulerException {
		if (scheduler != null) {
			scheduler.start();
		}
	}
	
	/**
	 * 开始一个定时任务
	 * 
	 * @param scheduler
	 * @param name
	 * @param cronexpression
	 * @param jobClass
	 * @param data
	 * @param status
	 * @throws SchedulerException
	 */
	private void addJob(org.quartz.Scheduler scheduler,String name, String cronexpression, Class<? extends Job> jobClass, Object data, String status)
			throws SchedulerException {
		JobKey jobKey = new JobKey(name, name);
		if (scheduler.checkExists(jobKey)) {
			scheduler.pauseJob(jobKey);
			scheduler.deleteJob(jobKey);
		}
		JobDataMap jobMap = new JobDataMap();
		if (data != null) {
			jobMap.put("data", data);
			jobMap.put("status", status);
		}
		JobDetail detail = JobBuilder.newJob(jobClass).withIdentity(jobKey).usingJobData(jobMap).build();
		CronTrigger trigger = TriggerBuilder.newTrigger().withIdentity(new TriggerKey(name + "trigger", name))
				.withSchedule(CronScheduleBuilder.cronSchedule(cronexpression)).build();
		scheduler.scheduleJob(detail, trigger);
	}
	
	
	/**
	 * 更新Job
	 * 
	 * @param scheduler
	 */
	private void jobStatusUpdate(org.quartz.Scheduler scheduler) {
		logger.info("【任务更新】 .... ....");
		
		List<DataSyncJobBean> list = null;
		if(dealtype == null || "".equals(dealtype) || "all".equals(dealtype)) {
			list = dao.querySyncJob();
		}else {
			list = dao.querySyncJobByType(dealtype);
		}
		if (list != null && list.size() > 0) {
			for (DataSyncJobBean syncJob : list) {
				syncJob.setCronExpression(JOB_CRON);
				String jobStatus = syncJob.getStatus();
				String jobName = syncJob.getName();
				
				// 如果是 有效的 job, 检查是否是已经执行的job，若未执行的job 直接加入 scheduler 中
				if ("0".equals(jobStatus)) {
					if (isJobRunning(scheduler, jobName)) {
						continue;
					}
					
					logger.info("【新增Job】 " + syncJob.getName());
					
					try {
						Class<? extends Job> jobClass = (Class<? extends Job>) Class.forName(JOB_CLASS_NAME);
						addJob(scheduler, syncJob.getName(), syncJob.getCronExpression(), jobClass, syncJob, jobStatus);
					} catch (Exception e) {
						logger.info("新增Job failed .... ....");
					}
				}
				
				// 如果是 无效的 job, 检查是否是已经执行的job，若是执行中的job 直接从 scheduler 中 删除
				if ("1".equals(jobStatus)) {
					if (isJobRunning(scheduler,jobName)) {
						logger.info("【删除Job】 " + syncJob.getName());
						
						try {
							JobKey jobKey = new JobKey(jobName, jobName);
							scheduler.pauseJob(jobKey);
							scheduler.deleteJob(jobKey);
							
							ShareVarUtil.setJobConsumerThreadNum(jobName, 0);
							ShareVarUtil.setJobSendThreadNum(jobName, 0);
						} catch (Exception e) {
							logger.info("删除Job failed .... ....");
						}
					}
				} else {
					continue;
				}
				
				
			}
		}
		
	}
	
	/**
	 * 
	 * 判断此任务是不是已经执行
	 * 
	 * @param scheduler
	 * @param name
	 * @return
	 */
	private boolean isJobRunning(org.quartz.Scheduler scheduler,String name) {
		boolean ret = false;
		JobKey jobKey = new JobKey(name, name);
		try {
			if (scheduler.checkExists(jobKey)) {
				ret = true;
			}
		} catch (SchedulerException e) {
			ret = false;
			logger.info("job checked error .... ....");
		}
		return ret;
	}
	
	
}
