package com.chinacreator.datasync.util;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.*; 

public class HttpInvoke {
	
	protected static Logger log = Logger.getLogger(HttpInvoke.class);
	
	private static class TrustAnyTrustManager implements X509TrustManager {  
		     
	   public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
		   
	   }  
	 
	   public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
		   
	   }  
	 
	   public X509Certificate[] getAcceptedIssuers() {  
	       return new X509Certificate[]{};  
	   }  
	}  
	     
	private static class TrustAnyHostnameVerifier implements HostnameVerifier {  
		public boolean verify(String hostname, SSLSession session) {  
		     return true;  
		}  
	}
	
	/**
	 * 处理https请求，超时时间默认为30秒
	 * @param method
	 * @param url
	 * @param content
	 * @param reqProperty
	 * @return
	 * @throws Exception 
	 */
	public static String sendhttpsReq(String method, String url, String content, Map reqProperty) throws Exception{
		return sendhttpsReq(method, url, content, reqProperty, 0);
	}
	
	/**
	 * 处理https请求
	 * @param method
	 * @param url
	 * @param content
	 * @param reqProperty
	 * @param timeout 超时时间， 如果小于或等于0，给默认值30秒
	 * @return
	 * @throws Exception 
	 */
	public static String sendhttpsReq(String method, String url, String content, Map reqProperty, int timeout) throws Exception{
		return sendhttpsReq(method, url, content, reqProperty, timeout, null);
	}
	
	/**
	 * 处理https请求
	 * @param method
	 * @param url
	 * @param content
	 * @param reqProperty
	 * @param timeout 超时时间， 如果小于或等于0，给默认值30秒
	 * @param tls TLS协议，为空是使用SSL
	 * @return
	 * @throws Exception 
	 */
	public static String sendhttpsReq(String method, String url, String content, Map reqProperty, int timeout, String tls) throws Exception{
		if(timeout <= 0){
			timeout = 30000; //如果超时时间小于或等于0，给默认值30秒
		}
		if(tls == null || "".equals(tls)){
			tls = "SSL";
		}
		StringBuffer buffer = new StringBuffer();
    	HttpsURLConnection connection = null;
    	BufferedReader reader = null;
    	OutputStream out = null;
		InputStream inputStream = null;
		InputStreamReader inputStreamReader = null;
    	try {
    		log.info("invokeurl=>"+url+"，reqProperty=>"+reqProperty+"，content=>"+content);
    		
			SSLContext sc = SSLContext.getInstance(tls);
			sc.init(null, new TrustManager[]{new TrustAnyTrustManager()}, new java.security.SecureRandom());  
			connection = (HttpsURLConnection) new URL(url).openConnection();
			connection.setSSLSocketFactory(sc.getSocketFactory());  
			connection.setHostnameVerifier(new TrustAnyHostnameVerifier());  
			connection.setRequestMethod(method);
			connection.setDoOutput(true);
			connection.setDoInput(true);
			//设置超时时间为30秒
			connection.setConnectTimeout(timeout);
			connection.setReadTimeout(timeout);
			
			if(reqProperty != null && reqProperty.size()>0){
				Set keySet = reqProperty.keySet();
				Iterator iterator = keySet.iterator();
				String key = "";
				String value = "";
				while(iterator.hasNext()){
					key = (String)iterator.next(); 
					value = (String)reqProperty.get(key);
					connection.setRequestProperty(key, value);
				}
			}else{
				//默认设置
				if("GET".equals(method)){
					connection.setRequestProperty("Content-type", "text/plain");
				}else{
					connection.setRequestProperty("Content-type", "application/json;charset=utf-8");
				}
			}
			
			connection.connect();
			out = connection.getOutputStream();
			out.write(content.getBytes("UTF-8"));
			out.flush();
			
			int responseCode = connection.getResponseCode();
			log.info("responseCode:"+responseCode);
			//小于400，从inputStream获取响应数据
			if(responseCode < HttpsURLConnection.HTTP_BAD_REQUEST){
				inputStream = connection.getInputStream();
			}else{//小于或等于400，从errorStream获取响应数据
				inputStream = connection.getErrorStream();
			}
			//inputStream = connection.getInputStream();
			inputStreamReader = new InputStreamReader(inputStream, "utf-8");
			reader = new BufferedReader(inputStreamReader);

			String line = "";
			while ((line = reader.readLine()) != null) {
				buffer.append(line);
			}
			reader.close();
		} catch (IOException e) {
			e.printStackTrace();
			log.error("调HTTP请求出现IO异常："+e.getMessage());
			throw e;
		} catch (Exception e){
			e.printStackTrace();
			log.error("调HTTP请求出现异常："+e.getMessage());
			throw e;
		} finally {
			if(inputStreamReader != null){
				try {
					inputStreamReader.close();
				} catch (IOException e) {
					inputStreamReader = null;
					log.error("释放inputStreamReader异常："+e.getMessage());
				}
			}
			if(inputStream != null){
				try {
					inputStream.close();
				} catch (IOException e) {
					inputStream = null;
					log.error("释放inputStream异常："+e.getMessage());
				}
			}
			if(out != null){
				try {
					out.close();
				} catch (IOException e) {
					out = null;
					log.error("释放out异常："+e.getMessage());
				}
			}
			if(reader != null){
				try {
					reader.close();
				} catch (IOException e) {
					reader = null;
					log.error("释放reader异常："+e.getMessage());
				}
			}
			if (connection != null) {
				connection.disconnect();
			}
		}
		return buffer.toString();
    }
	
	/**
	 * 发送http请求
	 * @param method POST/GET
	 * @param invokeurl  调用地址
	 * @param content 参数内容，GET请求不取值
	 * @return
	 * @throws Exception
	 */
	public static String sendhttpReq(String method, String invokeurl) throws Exception {
		return sendhttpReq(method, invokeurl, null, null, 0);
	}
	
	/**
	 * 发送http请求
	 * @param method POST/GET
	 * @param invokeurl  调用地址
	 * @param content 参数内容，GET请求不取值
	 * @param reqProperty
	 * @param timeout
	 * @return
	 * @throws Exception
	 */
	public static String sendhttpReq(String method, String invokeurl, String content, Map reqProperty, int timeout) throws Exception {
		if(timeout <= 0){
			timeout = 30000; //如果超时时间小于或等于0，给默认值30秒
		}
		URL url = null;
		HttpURLConnection connection = null;
		StringBuffer buffer = new StringBuffer();
    	BufferedReader reader = null;
    	DataOutputStream out = null;
		InputStream inputStream = null;
		InputStreamReader inputStreamReader = null;
		try {
		
			log.info("invokeurl=>"+invokeurl+"，reqProperty=>"+reqProperty);
			
			url = new URL(invokeurl);	
			
			connection = (HttpURLConnection) url.openConnection();
			if(reqProperty != null && reqProperty.size()>0){
				Set keySet = reqProperty.keySet();
				Iterator iterator = keySet.iterator();
				String key = "";
				String value = "";
				while(iterator.hasNext()){
					key = (String)iterator.next(); 
					value = (String)reqProperty.get(key);
					connection.setRequestProperty(key, value);
				}
			}else{
				//默认设置
				if("GET".equals(method)){
					connection.setRequestProperty("Content-type", "text/plain");
				}else{
					connection.setRequestProperty("Content-type", "application/json;charset=utf-8");
				}
			}
			connection.setDoOutput(true);
			connection.setDoInput(true);
			connection.setRequestMethod(method);
			connection.setUseCaches(false);
			//设置超时时间为30秒
			connection.setConnectTimeout(timeout);
			connection.setReadTimeout(timeout);
			connection.connect();
			
			if(!"GET".equals(method)){
				out = new DataOutputStream(connection.getOutputStream());
				out.write(content.getBytes("utf-8"));
				out.flush();
				out.close();
			}
			
			int responseCode = connection.getResponseCode();
			log.info("responseCode:"+responseCode);
			/***
			if(responseCode >= 300){
				//用于从错误条件（例如403）中检索数据
				inputStream = connection.getErrorStream();
			}else if(responseCode == 200) {
		    	inputStream = connection.getInputStream();
		    }
			***/
			//小于400，从inputStream获取响应数据
			if(responseCode < HttpsURLConnection.HTTP_BAD_REQUEST){
				inputStream = connection.getInputStream();
			}else{//小于或等于400，从errorStream获取响应数据
				inputStream = connection.getErrorStream();
			}
			
			//inputStream = connection.getInputStream();
			inputStreamReader = new InputStreamReader(inputStream, "utf-8");
			reader = new BufferedReader(inputStreamReader);
			String line = "";
			while ((line = reader.readLine()) != null) {
				buffer.append(line);
			}
			
			reader.close();			
		} catch (IOException e) {
			e.printStackTrace();
			log.error("调HTTP请求出现IO异常："+e.getMessage());
			throw e;
		} catch (Exception e){
			e.printStackTrace();
			log.error("调HTTP请求出现异常："+e.getMessage());
			throw e;
		} finally {
			if(inputStreamReader != null){
				try {
					inputStreamReader.close();
				} catch (IOException e) {
					inputStreamReader = null;
					log.error("释放inputStreamReader异常："+e.getMessage());
				}
			}
			if(inputStream != null){
				try {
					inputStream.close();
				} catch (IOException e) {
					inputStream = null;
					log.error("释放inputStream异常："+e.getMessage());
				}
			}
			if(out != null){
				try {
					out.close();
				} catch (IOException e) {
					out = null;
					log.error("释放out异常："+e.getMessage());
				}
			}
			if(reader != null){
				try {
					reader.close();
				} catch (IOException e) {
					reader = null;
					log.error("释放reader异常："+e.getMessage());
				}
			}
			if (connection != null) {
				connection.disconnect();
			}
		}
		return buffer.toString();
	}	
	
	/**
	 * 把map转换为url地址Params
	 * @param map 存储参数的map
	 * @return  返回完整的url地址
	 */
	public static String mapToUrl(Map<String,String> map){
		StringBuilder param = new StringBuilder("");
		if (map == null) {
			return "";
		}
		for(Map.Entry<String,String> entry: map.entrySet()){
			String key =  entry.getKey();
			String val =  entry.getValue();
			param.append(key).append("=").append(val).append("&");
		}
		//删去最后的&
		if (param.toString().endsWith("&")) {
			param.deleteCharAt(param.length() - 1);
		}
		return param.toString();
	}
	
	public static String sendhttpsReqAll(String method, String url, String content, Map reqProperty, int timeout, String tls) throws Exception{
		if(timeout <= 0){
			timeout = 30000; //如果超时时间小于或等于0，给默认值30秒
		}
		if(tls == null || "".equals(tls)){
			tls = "SSL";
		}
		StringBuffer buffer = new StringBuffer();
    	HttpsURLConnection connection = null;
    	BufferedReader reader = null;
    	OutputStream out = null;
		InputStream inputStream = null;
		InputStreamReader inputStreamReader = null;
    	try {
    		log.info("invokeurl=>"+url+"，reqProperty=>"+reqProperty+"，content=>"+content);
    		
			SSLContext sc = SSLContext.getInstance(tls);
			sc.init(null, new TrustManager[]{new TrustAnyTrustManager()}, new java.security.SecureRandom());  
			connection = (HttpsURLConnection) new URL(url).openConnection();
			connection.setSSLSocketFactory(sc.getSocketFactory());  
			connection.setHostnameVerifier(new TrustAnyHostnameVerifier());  
			connection.setRequestMethod(method);
			connection.setDoOutput(true);
			connection.setDoInput(true);
			//设置超时时间为30秒
			connection.setConnectTimeout(timeout);
			connection.setReadTimeout(timeout);
			
			if(reqProperty != null && reqProperty.size()>0){
				Set keySet = reqProperty.keySet();
				Iterator iterator = keySet.iterator();
				String key = "";
				String value = "";
				while(iterator.hasNext()){
					key = (String)iterator.next(); 
					value = (String)reqProperty.get(key);
					connection.setRequestProperty(key, value);
				}
			}
			
			connection.connect();
			if(content != null && content.length()>0){
				out = connection.getOutputStream();
				out.write(content.getBytes("UTF-8"));
				out.flush();
			}
			
			int responseCode = connection.getResponseCode();
			log.info("responseCode:"+responseCode);
			//小于400，从inputStream获取响应数据
			if(responseCode < HttpsURLConnection.HTTP_BAD_REQUEST){
				inputStream = connection.getInputStream();
			}else{//小于或等于400，从errorStream获取响应数据
				inputStream = connection.getErrorStream();
			}
			//inputStream = connection.getInputStream();
			inputStreamReader = new InputStreamReader(inputStream, "utf-8");
			reader = new BufferedReader(inputStreamReader);

			String line = "";
			while ((line = reader.readLine()) != null) {
				buffer.append(line);
			}
			reader.close();
		} catch (IOException e) {
			e.printStackTrace();
			log.error("调HTTP请求出现IO异常："+e.getMessage());
			throw e;
		} catch (Exception e){
			e.printStackTrace();
			log.error("调HTTP请求出现异常："+e.getMessage());
			throw e;
		} finally {
			if(inputStreamReader != null){
				try {
					inputStreamReader.close();
				} catch (IOException e) {
					inputStreamReader = null;
					log.error("释放inputStreamReader异常："+e.getMessage());
				}
			}
			if(inputStream != null){
				try {
					inputStream.close();
				} catch (IOException e) {
					inputStream = null;
					log.error("释放inputStream异常："+e.getMessage());
				}
			}
			if(out != null){
				try {
					out.close();
				} catch (IOException e) {
					out = null;
					log.error("释放out异常："+e.getMessage());
				}
			}
			if(reader != null){
				try {
					reader.close();
				} catch (IOException e) {
					reader = null;
					log.error("释放reader异常："+e.getMessage());
				}
			}
			if (connection != null) {
				connection.disconnect();
			}
		}
		return buffer.toString();
    }
}
