package com.chinacreator.datasync.util;

import java.util.HashMap;

public class ThreadLimitUtil {

	
	public static HashMap<String, Integer> threadMap = new HashMap<String, Integer>();
	
	//获取运行时间
	public static HashMap<String, Long> JobMap = new HashMap<String, Long>();
	
	//topic客户端消费存活监控记录
	public static HashMap<String, Long> consLiveMap = new HashMap<String, Long>();
	
	//topic客户端消费积压监控记录
	public static HashMap<String, String> recordsMap = new HashMap<String, String>();
	
	//存放每个topic配置的每次取数据条数
	public static HashMap<String, Integer> recordsMaxMap = new HashMap<String, Integer>();


	public static HashMap<String, Boolean> sharedQueueMap = new HashMap<String, Boolean>();
	
	//取kafkatopic线程最大运行时间，单位为秒，超过这个时间则重新启动线程
	//private static long topicRunMaxTime = 5 * 60;
	
	//处理数据job最大运行时间，单位 为少，超过这个时间则重新启动线程
	//private static long jobRunMaxTime = 10 * 60;
	
	//topic消费客户端存活记录刷新时间，单位 为秒，超过这个时间则重新设置
	private static long consLiveRefTime = 1 * 60;
	
	//topic积压记录最大时间值，单位 为秒
	private static long recordsRefTime = 5 * 60;
	
	/**
	 * 设置topic的最大取数据条数
	 * @param key
	 * @param _maxrecords
	 */
	public static void setRecordsMax(String key, String _maxrecords){
		try {
			if(!recordsMaxMap.containsKey(key)){
				recordsMaxMap.put(key, Integer.parseInt(_maxrecords));
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 获取topic的最大取数据条数
	 * @param key
	 * @return
	 */
	public static int getRecordsMax(String key){
		if(recordsMaxMap.containsKey(key)){
			return recordsMaxMap.get(key);
		}
		return 0;
	}
	
	/**
	 * 判断是否队列积压
	 * @param key
	 * @param maxrecords
	 * @return 积压返回true，反之为false
	 */
	public static boolean hasStockRecords(String key){
		boolean res = false;
		try {
			int recordsMax =0;
			if(recordsMaxMap.get(key) != null){
				recordsMax = recordsMaxMap.get(key);
			}
			if(recordsMap!=null && recordsMaxMap!=null) {
				if (recordsMap.containsKey(key) && recordsMax != 0) {
					long currtime = System.currentTimeMillis() / 1000;
					long resc = Long.parseLong(recordsMap.get(key).split("#")[1]);
					int records = Integer.parseInt(recordsMap.get(key).split("#")[0]);
					//当前时间减去最早记录的时间大于指定时间，说明积压记录超过了指定时间
					if ((records == recordsMax) && (currtime - resc > recordsRefTime)) {
						res = true;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return res;
	}

	/**
	 * 设置当前取值数
	 * @param key  topic+groupid
	 * @param records 当前取得数据条数
	 * @param maxrecords 每次取数据配置的最大记录条数
	 */
	public static void setRecordsMap(String key, int currrecords, int maxrecords){
		try {
			if(recordsMap.containsKey(key)){
				String recs = recordsMap.get(key).split("#")[0];
				if(Integer.parseInt(recs) == maxrecords && currrecords == maxrecords){
					//保存的记录数为最大值，且此次获取的数据条数与最大值相同，不处理
				}else{
					//更新缓存数据
					recordsMap.put(key, currrecords+"#"+System.currentTimeMillis()/1000);
				}
			}else{
				recordsMap.put(key, currrecords+"#"+System.currentTimeMillis()/1000);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 设置当前取值数
	 * @param key  topic+groupid
	 * @param records 当前取得数据条数
	 * 用于判断队列是否积压
	 * 配置每次取数据最大条数recordsMax为10，如果客户端每次取到数据都是10条，且连续超过5分钟，可以判断当前topic数据有积压
	 * 数据记录格式：key为topic##groupid，value为最后的取数据条数#记录时间
	 */
	public static void setRecordsMap(String key, int currrecords){
		try {
			int recordsMax = 0 ;
			if(recordsMaxMap.get(key) != null){
				recordsMax = recordsMaxMap.get(key);
			}
			if(recordsMap.containsKey(key)){
				String recs = recordsMap.get(key).split("#")[0];
				if(Integer.parseInt(recs) == recordsMax && currrecords == recordsMax){
					//保存的记录数为最大值，且此次获取的数据条数与最大值相同，不处理
				}else{
					//更新缓存数据
					recordsMap.put(key, currrecords+"#"+System.currentTimeMillis()/1000);
				}
			}else{//新增监控数据
				recordsMap.put(key, currrecords+"#"+System.currentTimeMillis()/1000);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 设置topic消费端的存活状态，如果超过最大刷新时间，则重新设置为当前时间
	 * @param key
	 * @return
	 */
	public static void setConsLive(String key){
		try {
			if(!consLiveMap.containsKey(key)){
				consLiveMap.put(key, System.currentTimeMillis()/1000);
			}else{
				long tmp = consLiveMap.get(key);
				long currtime = System.currentTimeMillis()/1000;
				//差值大于刷新时间，重新设置
				if((currtime - tmp) > consLiveRefTime){
					consLiveMap.put(key, currtime);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		//logger.info("key=>"+key+", value=>"+consLiveMap.get(key));
	}
	
	/***
	 * 判断topic消费端是否为存活状态
	 * @param key
	 * @param consLiveMaxTime 最大存活时间，单位为秒
	 * @return 存活返回true，消亡返回false
	 */
	public static boolean hasConsLive(String key, long consLiveMaxTime){
		boolean res = false;
		if(consLiveMap.containsKey(key)){
			long tmp = consLiveMap.get(key);
			long currtime = System.currentTimeMillis()/1000;
			//差值大于刷新时间，重新设置
			if((currtime - tmp) < consLiveMaxTime){
				res = true;
			}
		}
		//logger.info("res=>"+res+", key=>"+key+", consLiveMaxTime=>"+consLiveMaxTime+", consLiveMap.get(key)=>"+ consLiveMap.get(key) + ", currtime=>"+System.currentTimeMillis()/1000);
		return res;
	}
	
	/**
	 * 移除
	 * @param key
	 * @return
	 */
	public static boolean removeConsLive(String key){
		if(consLiveMap.containsKey(key)){
			consLiveMap.remove(key);
			return true;
		}
		return false;
	}
	
	
	/**
	 * 设置Job线程
	 * @param jobName
	 * @return true 设置成功，false 重复设置
	 */
	public static boolean setJobThread(String jobName){
		if(!JobMap.containsKey(jobName)){
			JobMap.put(jobName, System.currentTimeMillis()/1000);
			return true;
		}
		return  false;
	}
	
	/**
	 * 判断JOB是否大于最大运行时间
	 * @param jobName
	 * @return
	 */
	public static boolean hasGreatMaxRunTime(String jobname, String jobRunMaxTime){
		if(JobMap.containsKey(jobname)){
			long tmp = System.currentTimeMillis()/1000 - JobMap.get(jobname);
			if(tmp >=  Long.parseLong(jobRunMaxTime)){
				return true;
			}
		}
		return false;
	}
	
	/**
	 * 判断是否存在Job
	 * @param jobname
	 * @return
	 */
	public static boolean removeJobThread(String jobname){
		if(JobMap.containsKey(jobname)){
			JobMap.remove(jobname);
			return true;
		}
		return false;
	}
	
	/**
	 * 判断是否存在Job
	 * @param jobName
	 * @return
	 */
	public static boolean hasJobThread(String jobname){
		if(JobMap.containsKey(jobname)){
			return true;
		}
		return false;
	}
	
	/**
	 * 添加线程标识
	 * @param threadkey
	 * @return 返回当前KEY的线程数
	 */
	public static int setThread(String threadkey){
		if(!threadMap.containsKey(threadkey)){
			threadMap.put(threadkey, 1);
		}else{
			threadMap.put(threadkey, threadMap.get(threadkey)+1);
		}
		return threadMap.get(threadkey);
	}
	
	/**
	 * 根据KEY获取线程数
	 * @param threadkey
	 * @return
	 */
	public static int getThread(String threadkey){
		int res = 0;
		if(threadMap.containsKey(threadkey)){
			res = threadMap.get(threadkey);
		}
		return res;
	}
	
	/**
	 * 移除消费标识
	 * @param topic
	 * @return
	 */
	public static boolean removeKey(String threadkey){
		boolean res = false;
		threadMap.remove(threadkey);
		if(!threadMap.containsKey(threadkey)){
			res = true;
		}
		return res;
	}
}
